classdef GtAssembleVol3D < handle
% GtAssembleVol3D A class to assemble volumes
%
% Can be initialized as:
% assem = GtAssembleVol3D(varargin);
%
%   use parse_pv_pairs to handle various optional arguments, which:
%                     overlaps = 'zero' | 'summed' | 'conflict' | 'adaptive'
%              add_to_filename = .edf volume containing a set of existing grains
%       display_error_messages = [:logical]
%                        cache = GtSimpleCache object, in case you have a filled
%                                one, so you can avoid reloading things from
%                                disk
%             update_status_cb = Callback to update the status of the
%                                computation (used for instance in a gui). The
%                                said callback is executed every time the loop
%                                terminates the processing of an entry
%                     autosave = [:logical] Saves automatically after assembling
%
    properties
        parameters = [];
        localPar = [];

        volumes = [];

        redo = []; % Keeps track of the grains that had an error

        sigmas = []; % In case of twins
        symm=[]; % also in case of twins

        conflictStrategies = {'summed', 'zero', 'conflict', 'adaptive'};
    end

    methods (Access = public)
        function obj = GtAssembleVol3D(varargin)
        % GTASSEMBLEVOL3D/GTASSEMBLEVOL3D Constructor

            % Load Parameters
            obj.parameters = load('parameters');
            obj.parameters = obj.parameters.parameters;

            % Array of all the partial phase volumes
            obj.volumes.phases = {};
            % Complete volume: merge of all phases has two fields which really
            % hold the data:
            % - 'phases' which contains the information about the phaseIDs in
            %   all the voxels
            % - 'grains' which contains the information about the grainIDs in
            %   all the voxels
            obj.volumes.complete = [];
            obj.volumes.complete.phases = [];
            obj.volumes.complete.grains = [];

            % handle variables
            obj.localPar = GtAssembleVol3D.generateDefaults();

            obj.changeParams(varargin);
        end

        function rejected = changeParams(obj, arguments)
        % GTASSEMBLEVOL3D/CHANGEPARAMS Changes the local parameters, validates
        % them, and prepares for the correct usage of them
            if (nargout > 0)
                [obj.localPar, rejected] = parse_pv_pairs(obj.localPar, arguments);
            else
                obj.localPar = parse_pv_pairs(obj.localPar, arguments);
            end

            % not that "zeros" is ambiguous in Matlab because it is a function,
            % and in this context we normally use it as a string.  Use "zero"
            % instead, small bodge here in case this is not done.
            if ((~ischar(obj.localPar.overlaps) && (obj.localPar.overlaps == 0)) ...
                    || (ischar(obj.localPar.overlaps) && strcmpi(obj.localPar.overlaps, 'zeros')) )
                obj.localPar.overlaps = 'zero';
                disp('guessing that overlaps should be set to zero')
            end

            if (~ischar(obj.localPar.overlaps) ...
                    || ~ismember(obj.localPar.overlaps, obj.conflictStrategies))
                strats = obj.conflictStrategies;
                strats(2,:) = {'" | "'};
                error('ASSEMBLE:wrong_parameter', ...
                      'You should specify a valid "overlap" behavior: "%s" is not among ( "%s" )', ...
                      obj.localPar.overlaps, [strats{1:end-1}] )
            end

            % Initializing cache
            if (isempty(obj.localPar.cache))
                obj.localPar.cache = GtSimpleCache();
            end
            if (~obj.localPar.cache.isModel('sample'))
                grainDirsModel = fullfile('4_grains', 'sample.mat');
                obj.localPar.cache.createModel(grainDirsModel, 'sample');
            end
            if (~obj.localPar.cache.isModel('grain'))
                grainDirsModel = fullfile('4_grains', 'phase_%02d', 'grain_%04d.mat');
                obj.localPar.cache.createModel(grainDirsModel, 'grain');
            end

            if (obj.localPar.deal_with_twins)
                %sigma s: type, angle, axis {, brandon criteria}
                warning('gtnew_assemble_vol4_3D:sigmaAxisIndex', ...
                        ['sigma axis indices must be positive, normalised, ' ...
                         'and in descending order...'])
                pause(1)
                obj.sigmas = [ 3 60    1 1 1 0; ...
                               5 36.86 1 0 0 0; ...
                               7 38.21 1 1 1 0; ...
                               9 38.94 1 1 0 0];
                %add brandon criteria
                obj.sigmas(:, 6) = 15 * obj.sigmas(:, 1) .^ (-0.5);
                sigmaNorms = sqrt(sum(obj.sigmas(:, 3:5) .* obj.sigmas(:, 3:5), 2));
                %normalise axis
                obj.sigmas(:, 3:5) = obj.sigmas(:, 3:5) ...
                                     ./ sigmaNorms(:, [1 1 1]);
                % for this case, get the cubic symmetry operators too
                obj.symm = gtCrystGetSymmetryOperators('cubic');
            else
                disp('not dealing with twins')
            end
        end

        function assemblePhase(obj, phaseID)
        % GTASSEMBLEVOL3D/ASSEMBLEPHASE

            if (~exist('phaseID', 'var'))
                phaseID = 1;
            end

            [sampleInfo, list] = obj.initialiseAssembling(phaseID);

            if obj.parameters.acq.no_direct_beam
                disp('WARNING! No direct beam in the scan.')
                disp('  Revise sample bounding box before continuing!')
            end

            if (any(obj.parameters.acq.bb(3) == [0 1 NaN]) || ...
                any(obj.parameters.acq.bb(4) == [0 1 NaN]))
                disp('WARNING! Invalid value for ''parameters.acq.bb'' .')
            end

            % Let's initialize the volume
            volume = zeros(obj.parameters.acq.bb(3), ...
                           obj.parameters.acq.bb(3), ...
                           obj.parameters.acq.bb(4));

            if (obj.localPar.deal_with_twins)
                volume = obj.doAssemblePhaseVolumeTwins(volume, sampleInfo, phaseID, list);
            else
                volume = obj.doAssemblePhaseVolume(volume, sampleInfo, phaseID, list);
            end

            obj.volumes.phases{phaseID} = volume;

            if (obj.localPar.autosave)
                obj.savePhaseVolume(phaseID);
            end
        end

        function assembleAllPhases(obj)
        % GTASSEMBLEVOL3D/ASSEMBLEALLPHASES

            % Now let's load metadata
            sampleInfo = obj.localPar.cache.get('sample', {}, 'sample');

            % For all phases:
            for phaseID = 1:length(sampleInfo.phases)
                obj.assemblePhase(phaseID);
            end
        end

        function assembleInExistingVolume(obj, phaseID, filename)
        % GTASSEMBLEVOL3D/ASSEMBLEINEXISTINGVOLUME Discouraged function to write
        % a new grain map, to a previously existing grain map.

            if (~exist('phaseID', 'var'))
                phaseID = 1;
            end

            [sampleInfo, list] = obj.initialiseAssembling(phaseID);

            % look for the file
            if (~exist('filename', 'var'))
                filename = obj.localPar.add_to_filename;
            end
            absolute_file_path = fullfile(obj.parameters.acq.dir, ...
                                          '5_reconstruction', filename);

            if exist(filename, 'file')
                volume = edf_read(filename);
                % check if it's in the 5_reconstruction directory
            elseif exist( absolute_file_path, 'file')
                volume = edf_read(absolute_file_path);
                % otherwise, complain
            else
                error('ASSEMBLE:wrong_argument', ...
                      ['Cannot find the volume to add to: %s\ncheck the' ...
                       ' filename / path\n'], ...
                       filename)
            end

            % if we want to add grains to an existing volume, this is loaded
            % in. Then labels in the list are set to zero, before the grains
            % are "re-added" to the volume. I don't think this will deal
            % with overlapping grains correctly.
            if (max(volume(:)) > 0)
                for ii = 1:length(list)
                    volume(obj.volume == list(ii)) = 0;
                end
            end

            volume = obj.doAssemblePhaseVolume(volume, sampleInfo, phaseID, list);

            % Dumb saving.
            name = fullfile('5_reconstruction', filename);
            fprintf('writing volume %s\n', name);
            edf_write(volume, name, 'uint16');
        end

        function assembleCompleteVolume(obj)
        % GTASSEMBLEVOL3D/ASSEMBLEFINALVOLUME

            % Let's load metadata
            sampleInfo = obj.localPar.cache.get('sample', {}, 'sample');

            % Let's initialize the phase IDs volumes
            volumePIDs = zeros( obj.parameters.acq.bb(3), ...
                                obj.parameters.acq.bb(3), ...
                                obj.parameters.acq.bb(4), 'int16' );
            % Let's initialize the grains IDs volumes
            volumeGIDs = zeros( obj.parameters.acq.bb(3), ...
                                obj.parameters.acq.bb(3), ...
                                obj.parameters.acq.bb(4), 'int16' );

            fprintf('Processing phases: ');
            phasesNum = length(sampleInfo.phases);
            for ii = 1:phasesNum
                fprintf('%02d/%02d', ii, phasesNum);
                % no real need to cache phase volumes
                phaseVol = gtMATVolReader(sampleInfo.phases{ii}.volumeFile);
                grainMap = phaseVol;
                phaseVol = int16(phaseVol ~= 0) * ii;

                maskFile = fullfile('5_reconstruction', ...
                                    sprintf('phase_%02d_mask.mat', ii));
                if (exist(maskFile, 'file'))
                    maskVol = gtMATVolReader([maskFile ':mask']);
                    phaseVol = phaseVol .* int16(maskVol);
                    grainMap = grainMap .* int16(maskVol);
                end

                switch (obj.localPar.overlaps)
                    case {'zero', 'conflict', 'adaptive'}
                        overlappingPoints = ((volumePIDs ~= 0) & (phaseVol ~= 0));
                        volumePIDs = volumePIDs + phaseVol;
                        volumePIDs(overlappingPoints) = -1;

                        volumeGIDs = volumeGIDs + grainMap;
                        volumeGIDs(overlappingPoints) = -1;
                    case {'summed'}
                        volumePIDs = volumePIDs + phaseVol;
                        volumeGIDs = volumeGIDs + grainMap;
                end

                fprintf('\b\b\b\b\b');
            end
            fprintf('(%02d) Done.\n', phasesNum);

            % Shouldn't really happen, but it happens :)
            if (strcmpi(obj.localPar.overlaps, 'zero'))
                fprintf('Setting overlaps to 0..');
                numBefore = length(find(volumePIDs ~= 0));

                % set overlapping regions to zero - this could be another label colour
                volumePIDs(volumePIDs == -1) = 0;
                volumeGIDs(volumeGIDs == -1) = 0;

                numAfter = length(find(volumePIDs ~= 0));
                fprintf(' Done. (%d -> %d)\n', numBefore, numAfter);
            end

            obj.volumes.complete.phases = volumePIDs;
            obj.volumes.complete.grains = volumeGIDs;

            % Autosaving
            if (obj.localPar.autosave)
                obj.saveCompleteVolume();
            end
        end

        function savePhaseVolume(obj, phaseID, fileName)
        % GTASSEMBLEVOL3D/SAVEPHASEVOLUME
        %
            if (~exist('phaseID', 'var'))
                phaseID = 1;
            end
            if (~exist('fileName', 'var'))
                fileName = sprintf('phase_%02d_vol.mat', phaseID);
            end
            name = obj.checkAndSaveExistingFile(fileName);

            % Small trick to keep consistency in field name in the written file.
            outStruct = [];
            volField = 'vol';
            outStruct.(volField) = int16(obj.volumes.phases{phaseID});

            % We lock, to avoid other processes to write to this file:
            lock = GtLock(fullfile('4_grains', 'sample.mat'));
            lock.acquire();

            % Now let's load metadata
            sampleInfo = obj.localPar.cache.get('sample', {}, 'sample');
            obj.checkSampleInfo(sampleInfo, phaseID);

            % Store info about volume in the metadata
            sampleInfo.phases{phaseID}.volumeFile = [name ':' volField];
            obj.localPar.cache.set('sample', {}, sampleInfo, 'sample');
            % Flush metadata
            fprintf('Flushing sample.mat\n');
            obj.localPar.cache.flushObjectField('sample', {}, 'sample');

            % Let's release the lock
            lock.release();

            % Saving the data
            fprintf('Writing phase %02d volume to: %s\n', phaseID, name);
            save(name, '-struct', 'outStruct', volField, '-v7.3');
        end

        function saveCompleteVolume(obj, fileName)
        % GTASSEMBLEVOL3D/SAVECOMPLETEVOLUME
        %
            if (~exist('fileName', 'var'))
                fileName = 'volume.mat';
            end
            name = obj.checkAndSaveExistingFile(fileName);

            % We lock, to avoid other processes to write to this file:
            lock = GtLock(fullfile('4_grains', 'sample.mat'));
            lock.acquire();

            % Now let's load metadata
            sampleInfo = obj.localPar.cache.get('sample', {}, 'sample');

            % Store info about volume in the metadata
            sampleInfo.volumeFile = name;
            obj.localPar.cache.set('sample', {}, sampleInfo, 'sample');
            % Flush metadata
            fprintf('Flushing sample.mat\n');
            obj.localPar.cache.flushObjectField('sample', {}, 'sample');

            % Let's release the lock
            lock.release();

            % Saving the data
            fprintf('Writing complete volume to: %s\n', name);
            outStruct = obj.volumes.complete;
            save(name, '-struct', 'outStruct', 'phases', 'grains', '-v7.3');
        end

        function dilateCompleteVolume(obj, dilate_length, outFileName, inFileName)
        % GTASSEMBLEVOL3D/DILATECOMPLETEVOLUME
            if (~exist('outFileName', 'var'))
                outFileName = 'volume_dilated.mat';
            end
            name = obj.checkAndSaveExistingFile(outFileName);

            if (~exist('inFileName', 'var'))
                inFileName = fullfile('5_reconstruction', 'volume.mat');
            end

            compositeVol = load(inFileName);

            [uidVol, offset] = GtAssembleVol3D.convertCombinedVolumeToUIDs(compositeVol.phases, compositeVol.grains);

            % Try to see if there's a mask, and in case, use it
            maskFile = fullfile('5_reconstruction', 'volume_mask.mat');
            if (exist(maskFile, 'file'))
                maskVol = gtMATVolReader([maskFile ':vol']);

                fprintf('Doing %02d single dilation steps:\n', dilate_length)
                for step = 1:dilate_length
                    fprintf(' - %02d/%02d: ', step, dilate_length)
                    uidVol = gtDilateGrains(uidVol, 1, false);

                    fprintf('          Applying Absorption Mask.\n')
                    % Let's cut of the things out of the mask
                    newUidVol = zeros(size(uidVol), class(uidVol));
                    indexes = (maskVol ~= 0);
                    newUidVol(indexes) = uidVol(indexes);
                    uidVol = newUidVol;
                end
                fprintf('Done.\n')
            else
                uidVol = gtDilateGrains(uidVol, dilate_length, false);
            end

            [compositeVol.phases, compositeVol.grains] = GtAssembleVol3D.convertUIDsToCombinedVolume(uidVol, offset);

            save(name, '-struct', 'compositeVol', 'phases', 'grains', '-v7.3');
        end
    end

    methods (Access = private)
        function checkSampleInfo(~, sampleInfo, phaseID)
            if ((length(sampleInfo.phases) < phaseID) ...
                    || isempty(sampleInfo.phases{phaseID}))
                error('ASSEMBLE:wrong_argument', ...
                      'Phase: %02d, is not in the sample structure', phaseID);
            end
        end

        function list = getGrainsList(obj, grainsNum)
            if isempty(obj.localPar.list) || strcmpi(obj.localPar.list, 'all')
                list = 1:grainsNum;
            else
                list = obj.localPar.list;
            end
        end

        function finalName = checkAndSaveExistingFile(~, fileName)
        % GTASSEMBLEVOL3D/CHECKANDSAVEEXISTINGFILE Checks if the file exists and
        % saves it to the archive directory

            if (isempty(fileName))
                error('ASSEMBLE:wrong_argument', ...
                      'not saving data - to save, supply a filename')
            end
            finalName = fullfile('5_reconstruction', fileName);

            if (exist(finalName, 'file'))
                [~, tempFilename, tempExt] = fileparts(finalName);
                fileInfo = dir(finalName);
                destName = [tempFilename ...
                            datestr(fileInfo(1).datenum, '_yyyy-mm-dd_HH-MM-SS') ...
                            tempExt];
                if ~exist(fullfile('5_reconstruction', 'archive'),'dir')
                    [~,msg]=mkdir(fullfile('5_reconstruction', 'archive')); disp(msg)
                end
                copyfile(finalName, fullfile('5_reconstruction', 'archive', destName));
            end
        end

        function [sampleInfo, list] = initialiseAssembling(obj, phaseID)
        % GTASSEMBLEVOL3D/INITIALISEASSEMBLING Initialises the support
        % structures needed for assembly

            % Now let's load metadata
            sampleInfo = obj.localPar.cache.get('sample', {}, 'sample');
            % And check as possible
            obj.checkSampleInfo(sampleInfo, phaseID);
            % Grain Number
            grainsNum = length(sampleInfo.phases{phaseID}.grains);

            % handle list
            list = obj.getGrainsList(grainsNum);
        end

        function volume = doAssemblePhaseVolume(obj, volume, sampleInfo, phaseID, list)
        % GTASSEMBLEVOL3D/DOASSEMBLEPHASEVOLUME Function that effectively
        % assembles phase volume
            grainsNum = length(list);
            fprintf('Phase %02d: ', phaseID);
            for ii = list;
                fprintf('%04d/%04d', ii, grainsNum);

                if (~sampleInfo.phases{phaseID}.getSelected(ii))
                    fprintf('\nSkipping deselected grain: %04d\nPhase %02d: ', ii, phaseID)
                    continue;
                end

                try
                    vol = obj.localPar.cache.get('grain', {phaseID, ii}, 'seg');
                    volBBox = sampleInfo.phases{phaseID}.getBoundingBoxExtremes(ii);

                    volume = gtPlaceSubVolume(volume, vol, volBBox(1:3), ii, obj.localPar.overlaps);

                    % Rewind std output by 9 characters
                    fprintf('\b\b\b\b\b\b\b\b\b');
                catch mexc
                    gtPrintException(mexc, sprintf('\nGrain %d failed!!\nPhase %02d: ', ii, phaseID));
                    obj.redo = [obj.redo ii];
                end %try catch loop

                % Trigger callback (usually used in graphical applications)
                gtTriggerUpdateCallback(obj.localPar.update_status_cb);
            end  % for loop through all grains
            fprintf('(%04d) Done.\n', grainsNum);

            if (strcmpi(obj.localPar.overlaps, 'zero'))
                % set overlapping regions to zero - this could be another label colour
                volume(volume == -1) = 0;
            end
        end


        function volume = doAssemblePhaseVolumeTwins(obj, volume, sampleInfo, phaseID, list)
            % obj has the sigma as a property - not sure if useful like this
            % volume is zeros to build the new volume in
            % sample info includes bboxs and R vectors
            % phaseID - do we need?
            % list - grains to treat
            %
            % concept - place grain vol in a sample sized box
            % set undisputed voxels to the grainID
            % get a list of which grains overlap - test these for being twins
            % if they are twins, set the overlap to the grainID (principle of
            % largest first)

            % need some empty volumes to work in
            %    grainvolume=zeros(size(volume));
            %    grainvolume0=zeros(size(volume));

            % how many grains to treat?
            grainsNum = length(list);
            fprintf('Phase %02d: ', phaseID);

            % want to work in reverse size order
            grainsize = zeros(grainsNum, 1);
            for ii = 1:grainsNum
                try
                    vol = obj.localPar.cache.get('grain', {phaseID, list(ii)}, 'seg');
                    grainsize(ii) = length(find(vol(:)));
                catch mexc
                    % We ignore the error because it will come up again
                    % later
                    grainsize(ii) = 0;
                end
            end
            %sort - to do largest grain first
            [~, index] = sort(grainsize, 'descend');

            % loop through in descending size order
            for kk = 1:length(list)
                ii = index(kk);

                fprintf('%04d/%04d (grain %04d)', kk, grainsNum, ii);
                % skip deselected grains
                if (~sampleInfo.phases{phaseID}.getSelected(ii))
                    fprintf('\nSkipping deselected grain: %04d\nPhase %02d: ', ii, phaseID)
                    continue;
                end

                % read the grain volume
                try
                    vol = obj.localPar.cache.get('grain', {phaseID, ii}, 'seg');
                    volBBox = sampleInfo.phases{phaseID}.getBoundingBoxExtremes(ii);

                    % Getting the intersecting volume of complete volume and
                    % segmented volume
                    [limsVolume, limsGrain] = gtGetVolsIntersectLimits(size(volume), size(vol), volBBox(1:3));

                    % Getting the subvolumes -> so we operate all the costy
                    % operations on smaller volumes
                    subVolume = volume( limsVolume(1, 1):limsVolume(2, 1), ...
                                        limsVolume(1, 2):limsVolume(2, 2), ...
                                        limsVolume(1, 3):limsVolume(2, 3) );
                    subGrain = vol( limsGrain(1, 1):limsGrain(2, 1), ...
                                    limsGrain(1, 2):limsGrain(2, 2), ...
                                    limsGrain(1, 3):limsGrain(2, 3) );

                    %what grains overlap?
                    disputes = subVolume(find(subGrain));
                    disputes = unique(disputes);
                    disputes(find(disputes == 0 | disputes == -1)) = [];

                    % now, go through the disputed grains checking
                    % orientations
                    refRvector = sampleInfo.phases{phaseID}.R_vector(ii,:);

                    if ~isempty(disputes)
                        sigma = {};
                        for jj = 1:length(disputes)
                            % test for being a twin
                            testRvector = sampleInfo.phases{phaseID}.getR_vector(disputes(jj));
                            sigma{jj} = obj.twinTest(refRvector, testRvector); % obj.merge_angle?
                        end
                    else
                        sigma = {};
                    end

                    % Test if it is twin of another grain
                    if ~isempty(sigma)
                        % first, assign the non disputed part
                        subVolume(find(subVolume == 0 & subGrain)) = ii;

                        for jj = 1:length(disputes)
                            indexes = find((subVolume == disputes(jj)) & subGrain);

                            if ~isempty(sigma{jj})
                                % this is a twin.  Allow the new grain (smaller)
                                % to overwrite the older grain
                                subVolume(indexes) = ii;

%                                 twin_list_out(grainid, 1) = grainid;
%                                 twin_list_out(grainid, twincount+2) = list(j);
%                                 twincount=twincount+1;
%                             elseif (sigma == -1)
%                                 % merge the two grains together
%                                 grain_label = list(j);
%                                 output(find(output == grainid)) = grain_label;
%                                 disp(sprintf('changing label of grain %d to merge into %d', grainid, grain_label))
%
%                                 % record this in the twin list
%                                 twin_list_out(grainid, 2) = grain_label;
                            else
                                % this is not a twin, so it is disputed
                                switch (obj.localPar.overlaps)
                                    case {'zero', 'conflict', 'adaptive'}
                                        subVolume(indexes) = -1;
                                    case 'summed'
                                        subVolume(indexes) = subVolume(indexes) + ii;
                                    case 'assign'
                                        subVolume(indexes) = ii;
                                end
                            end
                        end

                        % Now let's get back the volume in the complete one
                        volume = gtPlaceSubVolume(volume, subVolume, volBBox(1:3), ...
                                                    0, 'assign');
                    else
                        % Ok, no conflict or no twin case -> let's treat it as
                        % normal case
                        volume = gtPlaceSubVolume(volume, vol, volBBox(1:3), ...
                                                    ii, obj.localPar.overlaps);
                    end

                    % Rewind std output by 9 characters
                    fprintf('\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b');
                catch mexc
                    gtPrintException(mexc, sprintf('\nGrain %d failed!!\nPhase %02d: ', ii, phaseID));
                    obj.redo = [obj.redo ii];
                end %try catch loop
            end
            fprintf('(%04d) Done.\n', grainsNum);

            if (strcmpi(obj.localPar.overlaps, 'zero'))
                % set overlapping regions to zero - this could be another label colour
                volume(volume == -1) = 0;
            end
        end

        function sigma = twinTest(obj, R1, R2) % , merge_angle)
        % GTASSEMBLEVOL3D/TWINTEST Tests if the two orientations are from two
        % twins. Input:
        % - R1 and R2: Rodriguez vectors of the two different orientations
        % - merge_angle: a threshold to decide if the angle is small
        % (currently not used)
            g1 = Rod2g(R1);
            g2 = Rod2g(R2);
            % need to search symmetry equivelents for the minimum misorientation
            rot_offset = zeros(1, 24);

            for k = 1:length(symm)
                g1equiv = g1 * obj.symm(k).g3;
                netg = g1equiv \ g2;
                rot_offset(k) = sum(diag(netg));
            end
            rot_offset = acos( (rot_offset - 1) / 2);

            [~, minIndex] = min(rot_offset);
            g1equiv = g1 * obj.symm(minIndex).g3;
            netg = g1equiv \ g2;

            mis_angle = (180 / pi) * rot_offset(minIndex);

            [eigVec, ~] = eig(netg);

            for p = 1:3
                if isreal(eigVec(:, p))  %find the real eigenvector
                    mis_axis = eigVec(:, p);
                end
            end

            sigma = []; % default sigma
            if ~isempty(mis_axis)
                mis_axis = sort(abs(mis_axis), 'descend');
                % determine sigma type
                test_angle = abs(mis_angle - obj.sigmas(:, 2)) < obj.sigmas(:, 6);
                test_axis = (180/pi)*abs(acos(dot(repmat(mis_axis',size(obj.sigmas,1),1), obj.sigmas(:,3:5), 2)))<obj.sigmas(:,6);
                sigma = obj.sigmas(find(test_angle & test_axis), 1);
            end
%             % add low angle (merge grains) criteria
%             if (mis_angle < merge_angle)
%                 sigma = -1;
%             end
        end % end of twinTest
    end

    methods (Static, Access = public)
        function conf = generateDefaults()
            % handle variables
            conf.overlaps = 'conflict';         % how to handle overlapping grains:
                                                        %   ('zero' | 'summed' | 'conflict' | 'adaptive')
            conf.add_to_filename = [];          % to add grains to a preexisting volume (saved as edf)
            conf.list = [];                     % if list is 'all', or [], load all grains.  Otherwise, only the list
            conf.display_error_messages = true; % show error messages
            conf.update_status_cb = [];         % Callback to update the status of anything
            conf.cache = [];                    % Grain volumes cache (to not reload all of them each time)
            conf.autosave = true;               % Saves automatically the volumes after assembling them
            conf.deal_with_twins = false;
            conf.symm=[];
        end

        function [newVol, offset] = convertCombinedVolumeToUIDs(phaseIDs, grainIDs, offset)
        % GTASSEMBLEVOL3D/CONVERTCOMBINEDVOLUMETOUIDS Converts from a twofold
        % matrix, into a unique ID matrix.
        % The conflicts are automatically set to zero.

            maxGrainID_1 = max(grainIDs(:)) +1;
            if (~exist('offset', 'var'))
                offset = maxGrainID_1;
            elseif (offset < maxGrainID_1)
                error('ASSEMBLE:wrong_argument', ...
                      'bad offset, there could be overlaps in IDs');
            end
            indexes = (grainIDs < 0);
            grainIDs(indexes) = 0;
            phaseIDs(indexes) = 0;
            newVol = grainIDs + (phaseIDs -1) * offset;
            % We need to kill the conflict regions if they exist, and the zeros.
            newVol(newVol < 0) = 0;
        end

        function [phaseIDs, grainIDs] = convertUIDsToCombinedVolume(uidVol, offset)
        % GTASSEMBLEVOL3D/CONVERTUIDSTOCOMBINEDVOLUME Converts back to a twofold
        % matrix, from a unique IDs matrix. The offset parameters is needed in
        % order to distinguish the phases.

            grainIDs = mod(uidVol, offset);
            uidVol = uidVol - grainIDs;
            phaseIDs = (floor(uidVol / offset) +1);
            phaseIDs(grainIDs == 0) = 0;
        end
    end
end
