function [f_handle, cb_handle] = gtDrawGrainUnitCells(grains, varargin)
%     [f_handle, cb_handle] = gtDrawGrainUnitCells(grains, varargin)
%     --------------------------------------------------------------
%
%     Draws a figure representing grain size, location and orientation by  
%     cubes or hexagonal prismes based on the their Rodrigues vectors.
%
%     INPUT:
%       grains      = grain structure from indexing <cell>
%
%     OPTIONAL INPUT (varargin):
%       grainids    = <double>       list of grain IDs of interest
%       hf          =                figure handle {}
%       cmap        = <double>       can be a component of the strain tensor
%                                    {grain.strain.strainT(3,3)} or a colour map
%       latticepar  = <double 1x6>   lattice parameters
%       labgeo      = <struct>       laboratory geometry as in parameters
%       phaseid     = <int>          phase number {1}
%       type        = <string>       unit cell type {'cubic'}/'hexagonal'
%       hlight      = <double>       grain id-s to be highlighted {}
%       scale       = <double>       number to scale grain size; empty to use bbox info in grain {}
%       view        = <double 1x2>   3D angular view {[0 90]}
%       showbar     = <logical>      flag to show the colourbar {false}
%       colormap    = <string>       type of the colormap 'jet',{'cool'},'gray','autumn','hsv',... 
%                                    only if it is not with true colours (RGB)
%                                    If needed, one can set the number of
%                                    levels: i.e. 'colormap','jet(10)'
%       strain      = <double>       scale factor for the strain values, if wanted {0}
%                                    zero to use the unstrained unit cell 
%       strainT     = <double>       component of the strainT tensor in 'strain'
%                                    {[3 3]}
%       zoom        = <double>       zoom in of this quantity {1.2}
%
%     OUTPUT:
%       f_handle    =                handle of the figure
%       cb_handle   =                handle of the colorbar
%
%
%     Version 004 15-11-2012 by LNervo
%        Use gtDrawSampleGeometry(labgeo) to create the graphics with axis and reference
%        system
%
%     Version 003 23-10-2012 by LNervo
%        Add varargin


% modify parameters/global settings
app.grainids    = [];
app.hf          = [];
app.cmap        = [];
app.latticepar  = [];
app.labgeo      = [];
app.phaseid     = 1;
app.type        = [];
app.hlight      = [];
app.scale       = [];
app.view        = [0 90];
app.showbar     = false;
app.colormap    = 'cool';
app.strain      = 0;
app.strainT     = [3 3];
app.zoom        = 1.2;

app = parse_pv_pairs(app, varargin);


if ~exist('grains','var') || isempty(grains)
   disp('Loading grain data from 4_grains/phase_##/index.mat ...')
   tmp = load(fullfile('4_grains',sprintf('phase_%02d',app.phaseid),'index.mat'),'grain');
   grains = tmp.grain;
end
if isempty(app.grainids)
    app.grainids = 1:length(grains);
end
if isempty(app.cmap) 
  if isfield(grains{1},'strain')
      app.cmap = gtIndexAllGrainValues(grains,'strain','strainT',app.strainT(1),app.strainT(2));
      disp('Using strainT(3,3) as colorscale')
      app.showbar = true;
  else
      app.cmap = zeros(length(grains),1);
      app.showbar = false;
  end
else
    if size(app.cmap,2)==1
        app.showbar = true;
    else  
        app.showbar = false;
    end
    if size(app.cmap,1)==1 && size(app.cmap,2)>1
        app.cmap = repmat(app.cmap,length(grains),1);
    end
    % remove bkg color
    if all(app.cmap(1,:) == 0)
        app.cmap(1,:) = [];
    end
end

grains = grains(app.grainids);
app.cmap = app.cmap(app.grainids,:);

parameters = [];
load('parameters.mat');
if isempty(app.latticepar)
    app.latticepar = parameters.cryst(app.phaseid).latticepar;
end
if isempty(app.labgeo)
    app.labgeo = parameters.labgeo;
end
if isempty(app.type)
    app.type = parameters.cryst(app.phaseid).crystal_system;
    if ~strcmpi(app.type,'cubic') && ~strcmpi(app.type, 'hexagonal')
        app.type = 'cubic';
        disp('Default shape for unit cells: cubic')
    end
end

if isempty(app.hf)
    hf = gtDrawSampleGeometry(app.labgeo.samenvrad, app.labgeo.samenvbot, app.labgeo.samenvtop);
else
    hf = app.hf;
end

hold on

nof_grains = length(grains);

pixelsize = (app.labgeo.pixelsizeu + app.labgeo.pixelsizev)/2;

if app.showbar
    colormap(app.colormap)
    disp(['Using colormap ' app.colormap '...'])
    mmin = min(app.cmap);
    mmax = max(app.cmap);
    ticks = mmin:(mmax-mmin)/10:mmax;
    cb = colorbar('FontSize',10,...
                  'YMinorTick','on');
end

% vertices and faces calculation
if strcmpi(app.type, 'cubic')
    data = gtCubicUnitCell('ratio',app.latticepar(3)/app.latticepar(1),'draw',false);
    vertices    = data.vertices(1:8,:);
    faces_sides = data.faces_sides;
    faces_end   = data.faces_end;
    hkl         = data.hkl;
elseif strcmpi(app.type, 'hexagonal')
    data = gtHexagonalUnitCell('ratio',app.latticepar(3)/app.latticepar(1),'draw',false);
    vertices    = data.vertices(1:12,:);
    faces_sides = data.faces_sides;
    faces_end   = data.faces_end;
    hkl         = data.hkil;
end

[Bmat, Amat] = gtCrystHKL2CartesianMatrix(app.latticepar);
pl_norm = gtCrystHKL2Cartesian(hkl, Bmat); % hkl plane normals in cryst reference

if app.strain~=0
    disp('Plotting strained unit cells for grains with strainT tensor...')
    disp(['Strain values are scaled by ' num2str(app.strain)])
end

% draw unit cells
for ii = 1:nof_grains
    if ismember(ii, app.hlight)
        hl = true;
    else
        hl = false;
    end
    sfPlotGrainCell(grains{ii}, vertices, faces_sides, hl, pixelsize, app.cmap(ii,:), app.scale, app.strain);
    sfPlotGrainCell(grains{ii}, vertices, faces_end, hl, pixelsize, app.cmap(ii,:), app.scale, app.strain);
end

axis equal;
zoom('out');
zoom(app.zoom)
zoom('off')
view(app.view);

f_handle = gcf;

if nargout == 2
    cb_handle = cb;
end

end % of main function


%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% SUB-FUNCTIONS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function sfPlotGrainCell(grain, vertices, faces, hl, pixelsize, cmap, scale, strainscale)

if isempty(scale)
    ga = 0.5*pixelsize*(grain.stat.bbxsmean + grain.stat.bbysmean)/2;
else
    ga = 0.5*pixelsize*(grain.stat.bbxsmean + grain.stat.bbysmean)/2*scale;
end

if isfield(grain,'centre')
	  gc = grain.centre;
else
  	gc = grain.center;
end

if isfield(grain, 'g')
    g_grain = grain.g;
else
    g_grain = Rod2g(grain.R_vector);
end
grain_vertices = ga*vertices*g_grain';
grain_vertices_st = repmat(gc,size(vertices,1),1) + (eye(3)*grain_vertices')';

if isfield(grain,'strain') && ~any(isnan(grain.strain.strainT(:))) && strainscale ~= 0
	  grain_vertices_st = grain_vertices_st + (grain.strain.strainT*strainscale*grain_vertices')';
end

facecolor = repmat(cmap,size(faces,1),1);

if hl
    patch('Vertices', grain_vertices_st, 'Faces', faces , ...
          'FaceVertexCData', facecolor, 'FaceColor', 'flat','EdgeColor','r','LineWidth',2);
else
    patch('Vertices', grain_vertices_st, 'Faces', faces , ...
          'FaceVertexCData', facecolor, 'FaceColor', 'flat');
end

end  % of sfPlotGrainCell
