function hf = gtIPFCmapHexagonalKey(N, saturate, drawing)
% GTIPFCMAPHEXAGONALKEY  Show inverse pole figure color space in the hexagonal SST
%
%     hf = gtIPFCmapHexagonalKey(N, saturate, drawing)
%     ------------------------------------------------
%
%     INPUT:
%       N        = <int>     Number of angular steps (quality of the plot) {30}
%       saturate = <logical> Saturate or not the color in the SST {true}
%       drawing  = <logical> Draw the patch with RGB colors {true}
%
%     OUTPUT:
%       hf       = <handle>  Resulting figure handle


if ~exist('N', 'var') || isempty(N)
    N = 30;
elseif N < 15
    warning('gtIPFCmapHexagonalKey:wrong_parameters', ...
        'Cannot use less than 15 angular steps, so using 15 steps.');
    N = 15;
end

if ~exist('drawing', 'var') || isempty(drawing)
    drawing = true;
end
if ~exist('saturate', 'var') || isempty(saturate)
    saturate = true;
end

hf = figure();
hold on;

% Work in radians
phimin =    0;
phimax = pi/6;
inc = phimax/N;

psimin =    0;
psimax = pi/2;

for phi=phimin:inc:(phimax-inc)     % Rotation around z-axis
    for psi=psimin:inc:(psimax-inc) % Out from z-axis to vector

        % For hexagonal colour map, we need two angles:
        % - phi (around z-axis)
        % - psi (from z-axis to vector)

        ratioR = 1 - psi/psimax;
        ratioB =     phi/phimax;

        red   = ratioR;
        green = (1 - ratioR) * (1 - ratioB);
        blue  = (1 - ratioR) * ratioB;        
        rgb = [red green blue];

        % Ensure that we stay in [0 1]
        rgb(rgb < 0) = 0;
        rgb(rgb > 1) = 1;

        if saturate
            % To improve colours distribution
            rgb = sqrt(rgb);
            % Saturate colours
            rgb = rgb./max(rgb);
        end

        % Radius distorted for stereographic proj
        r_patch = tan([psi/2 (psi+inc)/2]);
        [phi_patch, r_patch] = meshgrid([phi phi+inc]', r_patch);
        [x_patch, y_patch] = pol2cart(phi_patch, r_patch);

        if drawing
            patch(x_patch, y_patch, rgb, 'EdgeColor', 'none', 'Faces', [1 2 4 3]);
        end
    end
end

% Add poles and tart up like gtMakePoleFigure
set(gca, 'XTickLabel', '');
set(gca, 'YTickLabel', '');
set(gca, 'GridLineStyle', 'none');
set(gca, 'Ycolor', 'w');
set(gca, 'Xcolor', 'w');

axis tight
axis equal;
set(gca, 'xlim', [-0.05 1.05]);
set(gca, 'ylim', [-0.06 0.56]);

% Add a black border around triangle
% For hexagonal:
% - line from [0 0 0 1] to [2 -1 -1 0] (phi=0)
% - arc from [2 -1 -1 0] to [1 0 -1 0] (phi=[0 pi/6], psi=pi/2)
% - line from [0 0 0 1] to [1 0 -1 0] (phi=pi/6)
line_phi = (phimin:inc:phimax)';
line_r   = tan(psimax/2 * ones(size(line_phi)));

% Stereographic projection x-y
[line_x, line_y] = pol2cart(line_phi, line_r);
xp = [line_x; 0;  line_x(1)];
yp = [line_y; 0;  line_y(1)];
plot(xp, yp, 'k-');

% Add labels for the 3 corners poles
poles_psi = [psimin psimax psimax];
poles_phi = [phimin phimax phimin];
poles_r = tan(poles_psi/2);
[poles_x,poles_y] = pol2cart(poles_phi, poles_r);
plot(poles_x, poles_y, 'k*');

text(poles_x(1), poles_y(1), '[0 0 0 1]', ...
    'fontweight', 'bold','VerticalAlignment','top','HorizontalAlignment','left');
text(poles_x(2), poles_y(2), '[1 0 -1 0]', ...
    'fontweight', 'bold','VerticalAlignment','bottom','HorizontalAlignment','right');
text(poles_x(3), poles_y(3), '[2 -1 -1 0]', ...
    'fontweight', 'bold','VerticalAlignment','top','HorizontalAlignment','right');

set(gca, 'Visible', 'off');

% label and sample direction to plot
text(0,pi/6,'IPF', ...
    'fontweight', 'bold','VerticalAlignment','top','HorizontalAlignment','left');

end % end of function
