#!/usr/bin/python

import re
import fnmatch
import os
import sys

from dct_io_xml import DCTConf, DCTOutput

import dct_matlab_invocation

class FunctionsBuilder(object):

    submitMethods = [ 'OAR_make',
                    #  'condor_make' # Disabled since we moved to OAR
                    ]

    @staticmethod
    def getInstanceFromArgs(args, dct_dir = ""):
        if dct_dir is "":
            dct_dir = os.path.abspath(os.path.dirname(args[0]))
            dct_dir = os.path.join(dct_dir, os.path.pardir)
            dct_dir = os.path.abspath(dct_dir)
        confPath = os.path.join(dct_dir, "conf.xml")
        confPath = os.path.abspath(confPath)

        conf = DCTConf(confPath)
        matlab_path = conf.getMatlabPath()

        skip_next = False
        will_compile = True
        will_generate = True
        force_compile = False

        mfiles_to_consider = [ ]
        for index in range(len(sys.argv)):
            if skip_next is False:
                if sys.argv[index] == "-just-generate":
                    will_compile = False
                    will_generate = True
                if sys.argv[index] == "-just-compile":
                    will_compile = True
                    will_generate = False
                if sys.argv[index] == "-force-compile":
                    will_compile = True
                    force_compile = True
                if sys.argv[index] == "-c":
                    skip_next = True
                    if len(sys.argv) > (index-1):
                        mfiles_to_consider.append(sys.argv[index+1])
                    else:
                        raise ValueError("Not enough arguments for '-c' option")
                elif sys.argv[index] in ("--help", "-h"):
                    raise ValueError("")
                if sys.argv[index] == "-mp":
                    skip_next = True
                    if len(sys.argv) > (index-1):
                        matlab_path = sys.argv[index+1]
                    else:
                        raise ValueError("Not enough arguments for '-mp' option")
            else:
                skip_next = False

        return FunctionsBuilder(matlab_path = matlab_path, \
                                dct_dir = dct_dir, \
                                force_compile = force_compile, \
                                mfiles_to_consider = mfiles_to_consider, \
                                do_compile = will_compile, \
                                do_generate = will_generate)

    def __init__(self, matlab_path, dct_dir = os.getcwd(), \
                 script_dir = None, bin_dir = None, force_compile = False, \
                 mfiles_to_consider = [], do_compile = True, \
                 do_generate = True):
        self.dct_dir = dct_dir
        if script_dir is None:
            script_dir = os.path.join(self.dct_dir, 'bin', 'scripts')
        if bin_dir is None:
            bin_dir = os.path.join(self.dct_dir, 'bin', 'compiled')

        self.script_dir = script_dir
        self.bin_dir = bin_dir

        self.matlab_dir = matlab_path
        self.force_compile = force_compile
        self.mfiles_to_consider = mfiles_to_consider

        self.do_compile = do_compile
        self.do_generate = do_generate

        self.call_funcs = { }
        self.m_files = { }
        self.compile_records = { }

    def printHelp(self):
        launchname = os.path.basename(__file__)
        print("\"%s\" builds your mex files in the DCT dir: \"%s\"" %
              (launchname, self.dct_dir) )
        print(" Options:")
        print("  -h | --help : to show this help")
        print("  -v | --verbose : to show more detailed output")
        print("  -mp <matlab_dir> : to specify matlab's directory")
        print("  -c <filename> : to compile only a specific matlab function")
        print("  -force-compile : to force the compilation even if not needed")
        print("  -just-compile : to skip the generation of the script")
        print("  -just-generate : to skip the actual compilation")

    def _findFuncs(self):
        searchPatterns = [ ]
        for method in FunctionsBuilder.submitMethods:
            regexpString = '[^%]*\s*' + method + "\s*\(\s*'.*'"
            print('Regular expression: "' + regexpString + '"')
            regexp = re.compile(regexpString)
            searchPatterns.append(regexp)

        DCTOutput.printSubJob("Finding functions that may be compiled, from: '%s'" % self.dct_dir)
        filesList = [];
        for root, dir_name, file_names in os.walk(self.dct_dir):
            for file_name in file_names:
                if fnmatch.fnmatch(file_name, '*.m'):
                    filesList.append((root, file_name))

        for dir_path, file_name in filesList:
            filepath = os.path.join(dir_path, file_name)
            fid = open(filepath, 'r')
            filecontent = fid.readlines()
            fid.close()

            self.m_files[file_name[:-2]] = filepath
            for line in filecontent:
                for reg in searchPatterns:
                    matched = reg.search(line)
                    if matched:
                        function_name = matched.group().split("'")[1]
                        if filepath in self.call_funcs:
                            if function_name not in self.call_funcs[filepath]:
                                self.call_funcs[filepath].append(function_name)
                        else:
                            self.call_funcs[filepath] = [function_name]

        DCTOutput.printSubJob("These functions were found:")
        for file_name in self.call_funcs:
            DCTOutput.printSubSubJob('In file', file_name)
            for func in self.call_funcs[file_name]:
                print('      + Function: ' + func)
        print('')

    def _prepareFuncsList(self):
        DCTOutput.printSubJob("Adding information about functions that may not need to be compiled..")
        for caller in self.call_funcs:
            callerFuncName = os.path.basename(caller)[:-2]
            for function in self.call_funcs[caller]:
                if function not in self.compile_records.keys():
                    outpath = os.path.join(self.bin_dir, function)
                    try:
                        self.compile_records[function] = (outpath, self.m_files[function])
                    except KeyError:
                        print('    -> WARNING!! function: "' + function \
                              + '" is NOT a local function (may be from ID19) <-')
                        self.compile_records[callerFuncName][function] = (' ',' ')

    def _generateMCompileFile(self):
        if not os.path.exists(self.script_dir):
            DCTOutput.printSubJob("Creating script directory: '%s'" % self.script_dir)
            os.makedirs(self.script_dir)

        DCTOutput.printSubJob("Generating m-file that will compile those Functions..")
        script_content = [ "function compile(force, varargin)\n\n",
                           "mfiles = [];\n" ]
        for func in self.compile_records:
            out_file = self.compile_records[func][0]
            in_mfile = self.compile_records[func][1]
            script_content.append("mfiles.('%s') = struct( ...\n" % func)
            script_content.append("    'out_file', '%s', ...\n" % out_file)
            script_content.append("    'in_mfile', '%s' );\n" % in_mfile)
        script_content.append("\ngtCompileFunctions(force, mfiles, varargin)\nend\n")

        script_loc = os.path.join(self.script_dir, 'compile.m')
        DCTOutput.printSubJob("Writing '%s' script to disk.." % script_loc)
        fid = open(script_loc, "w")
        fid.writelines(script_content)
        fid.close()

    def compileFuncs(self):
        if not os.path.exists(self.bin_dir):
            DCTOutput.printSubJob('Creating binaries directory')
            os.makedirs(self.bin_dir)
        DCTOutput.printSubJob("Calling matlab..")

        extra_args = ['-nodesktop', '-nosplash']

        if self.force_compile is True:
            compile_args = "'force"
        else:
            compile_args = "'no-force"
        compile_args = "','".join([compile_args] + self.mfiles_to_consider) + "'"

        cmd = "compile(%s);quit;" % compile_args

        invoker = dct_matlab_invocation.DCTMatlabInvocation(dct_dir = self.dct_dir, \
                                                            work_dir = self.script_dir, \
                                                            extra_args = extra_args, \
                                                            extra_cmds = cmd, \
                                                            ctrl_c_kill = True)
        invoker.invoke()

        print("")

    def generateMfile(self):
        self._findFuncs()
        self._prepareFuncsList()
        self._generateMCompileFile()

    def buildMatlabFunctions(self):
        self.generateMfile()
        self.compileFuncs()


if __name__=="__main__":
    func_builder = FunctionsBuilder.getInstanceFromArgs(sys.argv)

    if func_builder.do_generate is True:
        func_builder.generateMfile()
    if func_builder.do_compile is True:
        func_builder.compileFuncs()


