#!/usr/bin/python

import fnmatch
import os
import sys
import subprocess
import string
import re

from dct_io_xml import DCTConf, DCTOutput

class MexBuilder(object):

    file_extensions = [ '*.c', '*.cxx', '*.cpp' ]

    @staticmethod
    def getInstanceFromArgs(args, dct_dir = ""):
        if dct_dir is "":
            dct_dir = os.path.abspath(os.path.dirname(args[0]))
            dct_dir = os.path.join(dct_dir, os.path.pardir)
            dct_dir = os.path.abspath(dct_dir)
        confPath = os.path.join(dct_dir, "conf.xml")
        confPath = os.path.abspath(confPath)

        conf = DCTConf(confPath)
        matlab_path = conf.getMatlabPath()

        skip_next = False
        force_compile = False
        verbose = False
        mfiles_to_consider = [ ]
        unclassified_args = [ ]

        for index in range(len(sys.argv)):
            if skip_next is False:
                if sys.argv[index] == "-force-compile":
                    force_compile = True
                elif sys.argv[index] in ("--verbose", "-v"):
                    verbose = True
                elif sys.argv[index] == "-c":
                    skip_next = True
                    if len(sys.argv) > (index+1):
                        mfiles_to_consider.append(sys.argv[index+1])
                    else:
                        raise ValueError("Not enough arguments to '-c' option")
                elif sys.argv[index] == "-mp":
                    skip_next = True
                    if len(sys.argv) > (index-1):
                        matlab_path = sys.argv[index+1]
                    else:
                        raise ValueError("Not enough arguments for '-mp' option")
                elif sys.argv[index] in ("--help", "-h"):
                    raise ValueError("")
                else:
                    unclassified_args.append(sys.argv[index])
            else:
                skip_next = False

        try:
            mex_info = conf.getMexFiles()
        except:
            DCTOutput.printWarning("Couldn't get mex files info. Using plain config.")
            mex_info = None

        return MexBuilder(matlab_path, dct_dir = dct_dir, \
                          extraArgs = unclassified_args, mex_info = mex_info, \
                          force_compile = force_compile, verbose = verbose, \
                          mfiles_to_consider = mfiles_to_consider)


    def __init__(self, matlab_path, dct_dir = None, outdir = None, \
                 extraArgs = [], mex_info = None, force_compile = False, \
                 verbose = False, mfiles_to_consider = []):
        """
         - force_compile: if true, compiles even if the file is recent
         - verbose: generates a lot of output
         - mfiles_to_consider: list mex files to compile (if empty, will compile all)
        """
        if dct_dir is None:
            dct_dir = os.getcwd()
        if outdir is None:
            outdir = os.path.join(dct_dir, 'bin', 'mex')
        self.dct_dir = dct_dir
        self.outdir = outdir

        self.matlab_dir = matlab_path

        self.c_files = { }
        self.mex_info = mex_info
        self.mfiles_to_consider = mfiles_to_consider

        self.force_compile = force_compile
        self.verbose = verbose

    def printHelp(self):
        launchname = os.path.basename(__file__)
        print("\"%s\" builds your mex files in the DCT dir: \"%s\"" %
              (launchname, self.dct_dir) )
        print("Usage: call it from within the directory of DCT. Be aware that" +
              " the DCT environment variable should be adjusted accordingly")
        print(" Options:")
        print("  -h | --help : to show this help")
        print("  -v | --verbose : to show more detailed output")
        print("  -c <filename> : to compile only a specific file (wildcards also apply)")
        print("  -mp <matlab_dir> : to specify matlab's directory")
        print("  -force-compile : to force the compilation even if not needed")

    def findMexs(self):
        """
        def findMexs():

        Finds the mex files (well, the C sources) and saves the files into the object
        """
        DCTOutput.printSubJob("Finding mex files, from: '%s'" % self.dct_dir)
        for root, dir_name, file_names in os.walk(self.dct_dir):
            for file_extension in self.file_extensions:
                for file_name in file_names:
                    if fnmatch.fnmatch(file_name, file_extension):
                        filepath = os.path.join(root, file_name)
                        self.c_files[file_name] = filepath
        DCTOutput.printSubJob("These files were found:")
        for file_name in self.c_files:
            DCTOutput.printSubSubJob('File', file_name + ' ( ' + self.c_files[file_name] + ' )')
        print('')

    def _buildMakeCmd(self, cmd, includes, lib_paths, libs, defines):
        if self.verbose is True:
            cmd.append('-v')
        for include in includes:
            cmd.append("-I" + include)
        for lib_path in lib_paths:
            cmd.append("-L" + lib_path)
        for lib in libs:
            cmd.append("-l" + lib)
        for define in defines:
            cmd.append("-D" + define)
        DCTOutput.printSubSubJob('File', cmd[1] + ' (cmd: ' + string.join(cmd, ' ') + ' )')
        subprocess.call(cmd)

    def _compileFile(self, file_name):
        """
        [Internal] def _compileFile(file_name):

        Tries to compile the given file_name.
        """
        file_path = self.c_files[file_name]
        compiler_path = os.path.join(self.matlab_dir, "bin", "mex") 
        cmd = [ compiler_path, file_path, '-outdir', self.outdir ]

        std_include = [ os.path.join(self.dct_dir, 'zUtil_Cxx', 'include') ]

        if self.mex_info is None:
            # No info at all into the xml file
#            DCTOutput.printWarning("Mex info not loaded.")
            self._buildMakeCmd(cmd, std_include, [], [], [])
        else:
            if self.mex_info.isExcluded(file_path) is False:
                props = self.mex_info.getFileProperties(file_path)
                if props is not None:
                    includes = std_include + props.get("includes")
                    self._buildMakeCmd(cmd, includes,
                                       props.get("lib_paths"),
                                       props.get("libs"),
                                       props.get("defines"))
                else:
                    # info found, but not about this file, so let's use standard params
#                    DCTOutput.printWarning("No Mex info on file: '%s'" % file_name)
                    self._buildMakeCmd(cmd, std_include, [], [], [])

    def compileFuncs(self):
        """
        def compileFuncs():

        Compiles the mex files into the list.
        """
        listOfErrors = [];

        if not os.path.exists(self.outdir):
            DCTOutput.printSubJob("Creating mex directory")
            os.makedirs(self.outdir)
        DCTOutput.printSubJob("Compiling mex files:")
        if ( len(self.mfiles_to_consider) is 0
             or ( len(self.mfiles_to_consider) is 1 and self.mfiles_to_consider[0] == 'all' ) ):
            # No preferences, so let's compile everything
            for file_name in self.c_files:
                self._compileFile(file_name)
        else:
            # Specific files
            for file_name in self.mfiles_to_consider:
                try:
                    self._compileFile(file_name)
                except KeyError as exc:
                    # Not found in the list, se let's check if it is a regular expression
                    try:
                        found = False
                        for cfile in self.c_files:
                            if re.search(file_name, cfile):
                                self._compileFile(cfile)
                                found = True
                        if found is not True:
                            DCTOutput.printError("Nothing found!")
                            raise exc
                    except:
                        errorMsg = 'ERROR. File not found: ' + file_name
                        DCTOutput.printError(errorMsg)
                        print(exc)
                        listOfErrors.append(errorMsg);
        print('')
        if len(listOfErrors) is not 0:
            print("Errors happened during compilation:")
            for errorMsg in listOfErrors:
                DCTOutput.printError('- %s' % errorMsg)

if __name__=="__main__":
    try:
        mex_builder = MexBuilder.getInstanceFromArgs(sys.argv)
        mex_builder.findMexs()
        mex_builder.compileFuncs()
    except ValueError as ex:
        if (len(ex.args) is not 0) and (ex.args[0] is not ""):
            DCTOutput.printError(ex.args)
        MexBuilder("").printHelp();
    except BaseException as ex:
        DCTOutput.printError(ex.args)
        exit(1);



