classdef GtGrainsManager < GtVolView
% GTGRAINSMANAGER
%
% You can pass extra behaviour options to the right click menu in the axes,
% using the argument syntax:
%   - 'plugins', {'menu_entry1', @callback1 [, 'menu_entry2', @callback2, ...]}
    properties
        selectedPhase = 0;
        selectedGrain = 0;

        plugins = {};

        separateGrainIDs = 0;

        dataDir;
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Public object API - Interaction with outside
    methods (Access = public)
        function obj = GtGrainsManager(dataDir, varargin)
        % GTGRAINSMANAGER/GTGRAINSMANAGER Constructor
            viewer_args = varargin;
            viewer_args(end+1:end+2) = {'f_title', 'Grains Manager'};

            vol = ones(2, 2, 2);
            vol(1:2, 1, 1) = 0;
            obj = obj@GtVolView(vol, viewer_args{:});

            if (~exist('dataDir', 'var') || isempty(dataDir))
                obj.dataDir = pwd();
            end

            % Initialise cache and metadata
            try
                grainsDir = fullfile(obj.dataDir, '4_grains');
                reconDir = fullfile(obj.dataDir, '5_reconstruction');

                sampleFileModel = fullfile(grainsDir, 'sample.mat');
                obj.cache.createModel(sampleFileModel, 'sample', 'error', 'always');

                grainDirsModel = fullfile(grainsDir, 'phase_%02d', 'grain_%04d.mat');
                obj.cache.createModel(grainDirsModel, 'grain');

                phaseVolumeModel = fullfile(reconDir, 'phase_%02d_vol.mat');
                obj.cache.createModel(phaseVolumeModel, 'phases');

                staticAbsVolumeModel = fullfile(reconDir, 'volume_absorption.mat');
                obj.cache.createModel(staticAbsVolumeModel, 'volume_absorption');

                staticVolumeModel = fullfile(reconDir, 'volume.mat');
                obj.cache.createModel(staticVolumeModel, 'volume');

                staticDilVolumeModel = fullfile(reconDir, 'volume_dilated.mat');
                obj.cache.createModel(staticDilVolumeModel, 'volume_dilated');

                obj.fillSidePanel();

                obj.reloadAssebledVolume();

                obj.updateStatus('Loaded.');
            catch mexc
                gtPrintException(mexc)
                d = errordlg(['Error: ' mexc.message], 'Fatal error', 'modal');
                uiwait(d, 10);
                obj.delete();
            end
        end

        function [infoStr, phaseID, grainID] = getGrainInfo(obj, point)
        % GTGRAINSMANAGER/GETGRAININFO Returns a string which contains
        % information about a specific grain, which resides in a specific voxel.
        %
        % The difference with the 'voxel details' action, is that this looks
        % into the assembled volume, while the other looks into the single grain
        % volumes.
        % So this give the iformation about the result of the assembling.

            [phaseID, grainID] = obj.getPhaseAndGrainOfPoint(point);

            if ((phaseID == 0) || (grainID == 0))
                infoStr = 'No grain selected';
            elseif ((phaseID < 0) || (grainID < 0))
                infoStr = 'Conflict region selected';
            else
                infoStr = obj.produceGrainInfo(phaseID, grainID);
            end
        end

        function selectDiffSpots(obj, phaseID, grainID)
        % GTGRAINSMANAGER/SELECTDIFFSPOTS Opens the guiGtMontage facility
        % function to select the diffraction spots in the given grain of the
        % selected phase

            if ((phaseID == 0) || (grainID == 0))
                infoStr = 'No grain selected';

                disp(infoStr)
                helpdlg(infoStr, 'Grain Diffspots');
            elseif ((phaseID < 0) || (grainID < 0))
                infoStr = 'Conflict region selected';

                disp(infoStr)
                helpdlg(infoStr, 'Grain Diffspots');
            else
                title = sprintf('Difspots of Grain %d, in Phase %d', grainID, phaseID);
                sample = obj.cache.get('sample', {}, 'sample');
                selected = sample.phases{phaseID}.getSelectedDiffspots(grainID);
                % Loading the stack of Diff Spots
                astraParams = obj.cache.get('grain', {phaseID, grainID}, 'proj');
                omegas = obj.cache.get('grain', {phaseID, grainID}, 'omega');
                spotsIDs = obj.cache.get('grain', {phaseID, grainID}, 'difspotID');
                % Launch Montage
                selected = guiGtMontage(astraParams.stack, selected, spotsIDs, omegas, title);
                % Save the selected
                sample.phases{phaseID}.setSelectedDiffspots(grainID, selected);
                obj.setSample(sample);
            end
        end

        function reloadAssebledVolume(obj, varargin)
        % GTGRAINSMANAGER/RELOADASSEBLEDVOLUME Reloads the assembled volumes and
        % displays it

            try
                switch (obj.conf.load_vol)
                    case 'phase'
                        phaseid = get(obj.conf.h_phase_edit, 'String');
                        phaseid = str2double(phaseid);
                        grainIDs = obj.cache.get('phases', {phaseid}, 'vol');
                    case 'dilated'
                        phaseIDs = obj.cache.get('volume_dilated', {}, 'phases');
                        grainIDs = obj.cache.get('volume_dilated', {}, 'grains');
                    case 'complete'
                        phaseIDs = obj.cache.get('volume', {}, 'phases');
                        grainIDs = obj.cache.get('volume', {}, 'grains');
                    otherwise
                        error('GUI:wrong_argument', ...
                              'Unknown kind of volume: "%s"', obj.conf.load_vol)
                end

                if (strcmpi(obj.conf.load_vol, 'phase'))
                    finalVol = grainIDs;
                else
                    obj.separateGrainIDs = max(grainIDs(:));

                    finalVol = zeros(size(phaseIDs), class(phaseIDs));
                    for ii = 1:max(phaseIDs(:))
                        indexes = find(phaseIDs == ii);
                        finalVol(indexes) = grainIDs(indexes) + obj.separateGrainIDs * (ii-1);
                    end

                    % Treat conflicts
                    finalVol((phaseIDs == -1) | (grainIDs == -1)) = -1;
                end

                if (ismember('CMap', varargin(1:2:end)))
                    obj.changeVol(finalVol, varargin{:});
                else
                    newArgs = [varargin {'CMap', 'random'}];
                    obj.changeVol(finalVol, newArgs{:});
                end
            catch mexc
                gtPrintException(mexc, 'Couldn''t load Volume file');
                uiwait(errordlg(mexc.message, 'Couldn''t load Volume file'), 10);
            end
        end

        function [point, values] = getPhaseAndGrainOfCurrentPointer(obj)
            point = round(obj.get3DPixelInfo());
            [phaseNum, grainNum] = obj.getPhaseAndGrainOfPoint(point);
            values = [phaseNum, grainNum];
        end

        function [phaseNum, grainNum] = getPhaseAndGrainOfPoint(obj, point)
        % GTGRAINSMANAGER/GETPHASEANDGRAINOFPOINT Returns the Phase ID and Grain
        % ID in the selected voxel

            voxel = round(point);
            phaseIDs = obj.cache.get('volume', {}, 'phases');
            phaseNum = phaseIDs(voxel(1), voxel(2), voxel(3));

            if (phaseNum == 0)
                grainNum = 0;
            else
                grainIDs = obj.cache.get('volume', {}, 'grains');
                grainNum = grainIDs(voxel(1), voxel(2), voxel(3));
            end
        end

        function grains = getVoxelDetails(obj, point)
        % GTGRAINSMANAGER/GETVOXELDETAILS Returns the grains (organized by
        % phase) that claim the voxel

            voxel = round(point);
            sample = obj.getSample();
            phasesNum = length(sample.phases);
            grains = cell(1, phasesNum);
            % Check per each phase, which grains claim the voxel
            for phaseID = 1:phasesNum
                bboxList = sample.phases{phaseID}.simpleSearchBBtouched(voxel);
                for jj = 1:length(bboxList)
                    grainID = bboxList(jj);
                    seg = obj.cache.get('grain', {phaseID, grainID}, 'seg');
                    shift = sample.phases{phaseID}.getBoundingBoxShift(grainID);
                    pos = (voxel - shift) + 1;
                    if (seg(pos(1), pos(2), pos(3)) ~= 0)
                        grains{phaseID} = [grains{phaseID} grainID];
                    end
                end
            end
        end

        function gotToGrain(obj, phaseID, grainID)
        % GTGRAINSMANAGER/GOTTOGRAIN Goes to the selected grain, into the volume
            sample = obj.getSample();
            bbox = sample.phases{phaseID}.getBoundingBoxExtremes(grainID);
            extremes = [bbox(obj.conf.ortDim) bbox(obj.conf.ortDim + 3)];
            if (obj.conf.slicendx < extremes(1) || obj.conf.slicendx > extremes(2))
                center = round(sum(extremes) / length(extremes));
                center = max(center, get(obj.conf.h_slider, 'Min'));
                center = min(center, get(obj.conf.h_slider, 'Max'));
                obj.conf.slicendx = center;
                set(obj.conf.h_slider, 'Value', obj.conf.slicendx);
            end
        end

        function setThreshold(obj, phaseID, grainID, threshold)
        % GTGRAINSMANAGER/SETTHRESHOLD Sets the threshold of a specific grain
            sample = obj.getSample();
            sample.phases{phaseID}.setThreshold(grainID, threshold);
            obj.setSample(sample);
        end

        function segmentGrain(obj, phaseID, grainID, threshold, center)
        % GTGRAINSMANAGER/SETTHRESHOLD Sets the threshold of a specific grain
            sample = obj.getSample();
            sample.phases{phaseID}.setThreshold(grainID, threshold);
            segmenter = GtThreshold();
            if (exist('center', 'var'))
                grain = segmenter.thresholdSingleGrainInFile(phaseID, grainID, threshold, center);
            else
                grain = segmenter.thresholdSingleGrainInFile(phaseID, grainID, threshold);
            end
            sample.phases{phaseID}.setBoundingBox(grainID, grain.segbb);
            obj.setSample(sample)
        end
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Public Callbacks - Interaction with GUI elements
    methods (Access = public)
        function displayDiffspots(obj, point)
        % GTGRAINSMANAGER/DISPLAYDIFFSPOTS Callback which displays the selection
        % gui for the diffspots of a specific grain, which was selected for the
        % spcific voxel.

            [phaseNum, grainNum] = obj.getPhaseAndGrainOfPoint(point);
            obj.selectDiffSpots(phaseNum, grainNum);
        end

        function updateVisualisedPhase(obj, phaseid)
        % GTGRAINSMANAGER/UPDATEVISUALISEDPHASE Updates the visualised picture,
        % depending on the selected phase.

            try
                if (~exist('phaseid', 'var'))
                    phaseid = get(obj.conf.h_phase_edit, 'String');
                    phaseid = str2double(phaseid);
                else
                    set(obj.conf.h_phase_edit, 'String', num2str(phaseid));
                end
                obj.updateStatus(sprintf('Visualised phase: %d', phaseid));
                obj.selectedPhase = phaseid;
                obj.updateVisualisedGrain(0);
            catch mexc
                gtPrintException(mexc, 'Couldn''t change Phase ID');
            end
        end

        function updateVisualisedGrain(obj, grainid)
        % GTGRAINSMANAGER/UPDATEVISUALISEDGRAIN Updates the visualised picture,
        % depending on the selected phase and grain IDs.

            try
                if (~exist('grainid', 'var'))
                    grainid = get(obj.conf.h_grain_edit, 'String');
                    grainid = str2double(grainid);
                else
                    set(obj.conf.h_grain_edit, 'String', num2str(grainid));
                end
                if (obj.selectedPhase == 0)
                    set(obj.conf.h_grain_edit, 'Enable', 'off');
                    set(obj.conf.h_grain_butt, 'Enable', 'off');
                else
                    obj.updateStatus(sprintf('Visualised grain: %d', grainid));
                    set(obj.conf.h_grain_edit, 'Enable', 'on');
                    set(obj.conf.h_grain_butt, 'Enable', 'on');
                    if (grainid ~= 0)
                        obj.gotToGrain(obj.selectedPhase, grainid);
                    end
                end
                obj.selectedGrain = grainid;
                obj.updateDisplay();
            catch mexc
                gtPrintException(mexc, 'Couldn''t change Grain ID');
            end
        end

        function displayVolumeInfo(obj)
        % GTGRAINSMANAGER/DISPLAYVOLUMEINFO Displays Sample's and phases' volume
        % info

            sample = obj.getSample();
            guiSample = GtGuiSample(sample);
            newSample = guiSample.join();
            if (~isempty(newSample))
                obj.setSample(newSample);
                obj.updateStatus('Updated sample information.');
            end
        end

        function displayGrainInfo(obj, point)
        % GTGRAINSMANAGER/DISPLAYGRAININFO Displays a gui about the selected
        % grain, with the possibility to inspect its properties and interact
        % with it.

            [infoStr, phaseID, grainID] = obj.getGrainInfo(point);
            disp(infoStr);

            if ((phaseID < 1) || (grainID < 1))
                d = helpdlg(infoStr, 'Grain Info');
                uiwait(d);
            else
                d = dialog('Name', 'Grain Info', 'Resize', 'on', ...
                           'WindowStyle', 'normal');
                [phaseID, grainID] = obj.getPhaseAndGrainOfPoint(point);
                grainPanel = obj.makeGrainPanel(d, phaseID, grainID);
            end
        end

        function displayVoxelDetails(obj, point)
        % GTGRAINSMANAGER/DISPLAYVOXELDETAILS Displays a gui which gives a
        % perspective of the given voxel: grains which claim it, et cetera

            obj.updateStatus(sprintf('Display details of voxel (%d, %d, %d)..', ...
                                     round(point)));
            drawnow;

            dlgBlock = [];
            dlgBlock.d = dialog('Name', 'Voxel Details', 'Resize', 'on', ...
                                'WindowStyle', 'normal');
            dlgBlock.vert_boxes = uiextras.VBox('Parent', dlgBlock.d);
            dlgBlock.phases_boxes = {};

            finalGrains = obj.getVoxelDetails(point);
            visualize = false;

            sample = obj.getSample();
            % Check per each phase, which grains claim the voxel
            for phaseID = 1:length(sample.phases)
                if (~isempty(finalGrains{phaseID}))
                    dlgBlock.phases_boxes{phaseID} = obj.makePhasePanel(...
                        dlgBlock.vert_boxes, phaseID, [finalGrains{phaseID}]);
                    visualize = true;
                end
            end

            if (~visualize)
                delete(dlgBlock.d);
                dlgBlock.d = helpdlg('No grains found!', 'Voxel Details');
            end
            obj.updateStatus('Done.');
            drawnow;
        end

        function toggleSelected(obj, src, phaseID, grainID)
        % GTGRAINSMANAGER/TOGGLESELECTED Toggles the selected status of a grain

            isSelected = (get(src, 'Value') == get(src, 'Max'));
            if (isSelected)
                set(src, 'String', 'Active');
            else
                set(src, 'String', 'Not Active');
            end
            sample = obj.getSample();
            sample.phases{phaseID}.setSelected(grainID, isSelected);
            obj.setSample(sample);
        end

        function changeGrainThreshold(obj, phaseID, grainID)
        % GTGRAINSMANAGER/CHANGEGRAINTHRESHOLD Launches a gui object to adjust
        % the threshold of a specific grain.

            sample = obj.getSample();
            oldThr = sample.phases{phaseID}.getThreshold(grainID);
            thr = GtGuiThresholdGrain(obj.cache.get('grain', {phaseID, grainID}, 'vol'));

            thr.setSavingFunction(@(thresh)setThreshold(obj, phaseID, grainID, thresh));
            thr.setSegmentFunction(@(thresh, center)segmentGrain(obj, phaseID, grainID, thresh, center));

            thr.changeThreshold(oldThr);
        end

        function viewDeselectedGrains(obj)
        % GTGRAINSMANAGER/VIEWDESELECTEDGRAINS Launches a gui which let's you
        % inspect and interact with the deselected grains

            obj.updateStatus('Visualizing Deselected Grains..');
            drawnow;

            dlgBlock = [];
            dlgBlock.d = dialog('Name', 'Deselected Grains', 'Resize', 'on', ...
                                'WindowStyle', 'normal');
            dlgBlock.vert_boxes = uiextras.VBox('Parent', dlgBlock.d);
            dlgBlock.phases_boxes = {};

            visualize = false;
            sample = obj.getSample();
            % Check per each phase, which grains claim the voxel
            for phaseID = 1:length(sample.phases)
                deselected = sample.phases{phaseID}.getDeselectedGrains();
                if (~isempty(deselected))
                    dlgBlock.phases_boxes{phaseID} = obj.makePhasePanel(...
                        dlgBlock.vert_boxes, phaseID, deselected);
                    visualize = true;
                end
            end

            if (~visualize)
                delete(dlgBlock.d);
                dlgBlock.d = helpdlg('No deselected grains found!', ...
                                     'Deselected Grains');
            end
            obj.updateStatus('Done.');
            drawnow;
        end

        function launchReconSegGUI(obj)
        % GTGRAINSMANAGER/LAUNCHASSEMBLEGUI Launches the assembling interface

            obj.updateStatus('Launching Reconstructor..');
            recon = GtGuiReconstructAndSegment();
            obj.updateStatus('Done.');
        end

        function launchAssembleGUI(obj)
        % GTGRAINSMANAGER/LAUNCHASSEMBLEGUI Launches the assembling interface

            obj.updateStatus('Launching Assembler..');
            assemble = GtGuiAssembleVol3D('cache', obj.cache);
            obj.updateStatus('Done.');
        end

        function checkVolumesUpdate(obj)
        % GTGRAINSMANAGER/CHECKVOLUMESUPDATE Checks wether the volumes are up to
        % date with the grain segmented volumes

            try
                sample = obj.getSample();
                volumeFile = fullfile(obj.dataDir, sample.volumeFile);
                if (exist(volumeFile, 'file'))
                    volumeUpToDate = true;

                    fileInfo = dir(volumeFile);
                    volumeDateNum = fileInfo(1).datenum;
                else
                    volumeUpToDate = false;
                    volumeDateNum = 0;
                end
                phasesNum = length(sample.phases);
                for phaseID = 1:phasesNum
                    [upToDate, phaseDateNum] = obj.isPhaseUptodate(phaseID);
                    if (upToDate)
                        set(obj.conf.h_phases_labels{phaseID}, 'ForegroundColor', 'g');
                        if (volumeUpToDate && (phaseDateNum > volumeDateNum))
                            volumeUpToDate = false;
                        end
                    else
                        set(obj.conf.h_phases_labels{phaseID}, 'ForegroundColor', 'r');
                        volumeUpToDate = false;
                    end
                end
                if (volumeUpToDate)
                    set(obj.conf.h_volume_label, 'ForegroundColor', 'g');

                    % XXX - Should make it a field of GtSample
                    fileInfo = dir(fullfile(obj.dataDir, '5_reconstruction', 'volume_dilated.mat'));
                    if (~isempty(fileInfo) && volumeDateNum < fileInfo(1).datenum)
                        set(obj.conf.h_dil_volume_label, 'ForegroundColor', 'g');
                    else
                        set(obj.conf.h_dil_volume_label, 'ForegroundColor', 'r');
                    end
                else
                    set(obj.conf.h_volume_label, 'ForegroundColor', 'r');
                    set(obj.conf.h_dil_volume_label, 'ForegroundColor', 'r');
                end
            catch mexc
                errorTitle = 'Failed to check volume updates.';
                gtPrintException(mexc, errorTitle);
            end
        end

        function displayForwardSim(obj, phaseID, grainID)
            grain = [];
            grain.full = obj.cache.get('grain', {phaseID, grainID}, 'full');
            grain.ondet = obj.cache.get('grain', {phaseID, grainID}, 'ondet');
            grain.flag = obj.cache.get('grain', {phaseID, grainID}, 'flag');
            grain.allblobs = obj.cache.get('grain', {phaseID, grainID}, 'allblobs');
            grain.id = grainID;
            grain.phaseid = phaseID;
            gtShowFsim(grain);
        end

        function toggleAbsorptionVolume(obj)
            try
                sample = obj.getSample();
                if (isempty(obj.conf.overlay))
                    if (isempty(sample.absVolFile))
                        error('GUI:GtGrainsManager:wrong_argument', ...
                                'Absorption volume file not specified!')
                    end
                    absFile = [sample.absVolFile ':abs'];
                    obj.changeOverlay(absFile);
                    newValueAbsVol = get(obj.conf.h_abs_butt, 'Max');
                else
                    obj.changeOverlay([]);
                    newValueAbsVol = get(obj.conf.h_abs_butt, 'Min');
                end
                set(obj.conf.h_abs_butt, 'Value', newValueAbsVol);
            catch mexc
                gtPrintException(mexc)
                d = errordlg(['Error: ' mexc.message], mexc.identifier, 'modal');
                uiwait(d);
            end
        end

        function setVolumeToLoad(obj, volume_str)
            if (ismember(volume_str, {'phase', 'complete', 'dilated'}))
                obj.conf.load_vol = volume_str;
            else
                error('GUI:wrong_argument', ...
                      'No such kind of volume "%s"', volume_str);
            end
            maxRadio = get(obj.conf.h_c_vol_butt, 'Max');
            minRadio = get(obj.conf.h_c_vol_butt, 'Min');

            set(obj.conf.h_d_vol_butt, 'Value', minRadio);
            set(obj.conf.h_c_vol_butt, 'Value', minRadio);
            set(obj.conf.h_p_vol_butt, 'Value', minRadio);
            switch (obj.conf.load_vol)
                case 'phase'
                    set(obj.conf.h_p_vol_butt, 'Value', maxRadio);
                case 'complete'
                    set(obj.conf.h_c_vol_butt, 'Value', maxRadio);
                case 'dilated'
                    set(obj.conf.h_d_vol_butt, 'Value', maxRadio);
            end
        end

        function showQueryDialog(obj)
            dlgBlock = [];
            dlgBlock.d = dialog('Name', 'Query Grains', 'Resize', 'on', ...
                                'WindowStyle', 'normal');
            dlgBlock.p = obj.produceQueryPanel(dlgBlock.d);
        end

        function launchQuery(obj, panelBlk, queryType)
            % Let's delete what's already in the panel
            if (isstruct(panelBlk.result_panel) ...
                    && isfield(panelBlk.result_panel, 'Children'))
                prevRes = panelBlk.result_panel.Children;
                delete(prevRes);
                panelBlk.result_panel.Children = [];
                drawnow;
            end

            sample = obj.getSample();
            switch (queryType)
                case 'completeness'
                    value = str2double(get(panelBlk.h_query.completeness.edit, 'String'));
                    grainIDs = sample.getGrainsWithCompletenessLessThan(value);
                case 'difspots'
                    value = str2double(get(panelBlk.h_query.difspots.edit, 'String'));
                    grainIDs = sample.getGrainsWithDifspotsNumLessThan(value);
            end

            panelBlk.res_vert_boxes = uiextras.VBox('Parent', panelBlk.result_panel);
            for phaseID = 1:length(sample.phases)
                if (~isempty(grainIDs{phaseID}))
                    obj.makePhasePanel(panelBlk.res_vert_boxes, phaseID, grainIDs{phaseID});
                end
            end
            drawnow;
        end

        function reconstructGrain(obj, phaseID, grainID)
            doneLocally = gtReconstructGrains(grainID, grainID, obj.dataDir, phaseID);
            if (~doneLocally)
                warndlg('The job has been scheduled on OAR, there may be some delay', ...
                        'Not a GPU Machine!');
            end
        end

        function viewSegmentedGrain(obj, phaseID, grainID)
            grainSeg = obj.cache.get('grain', {phaseID, grainID}, 'seg');
            viewer = GtVolView(grainSeg);
        end
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Internal methods - Addressable from derived classes
    methods (Access = protected)
        function initParams(obj, arguments)
        % GTGRAINSMANAGER/INITPARAMS Preprocessing the arguments, before sending
        % them to GtVolView. It extracts the plugins from the arguments

            % Let's catch plugins
            pluginsCatcher = [];
            pluginsCatcher.plugins = {};
            [pluginsCatcher, viewer_args] = parse_pv_pairs(pluginsCatcher, arguments);

            obj.plugins = GtGrainsManager.checkMenuParams(pluginsCatcher.plugins);
            disp('Additional Plugins, specified by the user')
            disp(obj.plugins)

            initParams@GtVolView(obj, viewer_args);
        end

        function initGui(obj)
        % GTGRAINSMANAGER/INITGUI Inherited method that builds the interface

            initGui@GtVolView(obj)
            % Let's disable zoom, while configuring
            zoomOp = zoom(obj.conf.h_figure);
            zoomState = get(zoomOp, 'Enable');
            zoom(obj.conf.h_figure, 'off');

            obj.conf.main_g_boxes = uiextras.HBox('Parent', obj.conf.currentParent);

            obj.conf.center_g_boxes = uiextras.VBox('Parent', obj.conf.main_g_boxes);
            obj.conf.h_viewer_panel = uipanel('BorderWidth', 4, ...
                                              'BorderType', 'line', ...
                                              'Parent', obj.conf.center_g_boxes);
            obj.conf.lower_g_boxes = uiextras.HBox('Parent', obj.conf.center_g_boxes);

            set(obj.conf.center_g_boxes, 'Sizes', [-1 20]);

            obj.conf.side_g_boxes = uiextras.VButtonBox('Parent', obj.conf.main_g_boxes);

            set(obj.conf.main_g_boxes, 'Sizes', [-1 140]);

            obj.conf.h_grain_status = uicontrol('Style', 'text', ...
                                             'Parent', obj.conf.lower_g_boxes, ...
                                             'HorizontalAlignment', 'left');

            % Phase selection
            obj.conf.h_phase_label = uicontrol('Style', 'text', ...
                                             'Parent', obj.conf.lower_g_boxes, ...
                                             'HorizontalAlignment', 'left');
            obj.conf.h_phase_edit = uicontrol('Style', 'edit', ...
                                              'Parent', obj.conf.lower_g_boxes);
            obj.conf.h_phase_butt = uicontrol('Style', 'pushbutton', ...
                                              'String', 'Apply', ...
                                              'Parent', obj.conf.lower_g_boxes);
            % End Phase selection

            % GrainSelection
            obj.conf.h_grain_label = uicontrol('Style', 'text', ...
                                             'Parent', obj.conf.lower_g_boxes, ...
                                             'HorizontalAlignment', 'left');
            obj.conf.h_grain_edit = uicontrol('Style', 'edit', ...
                                              'Parent', obj.conf.lower_g_boxes);
            obj.conf.h_grain_butt = uicontrol('Style', 'pushbutton', ...
                                              'String', 'Apply', ...
                                              'Parent', obj.conf.lower_g_boxes);
            % End Grain selection

            set(obj.conf.lower_g_boxes, 'Sizes', [-1, 120, 40, 50, 120, 40, 50]);
            set(obj.conf.main_boxes, 'Parent', obj.conf.h_viewer_panel);

            % Reset back the zoom to the previous state (before the inclusion)
            set(zoomOp, 'Enable', zoomState);

            obj.conf.load_vol = 'complete';

            pixelInfoPattern = '(%d, %d, %d) Phase: %d, Grain %d';
            cbk = @(x, y)getPhaseAndGrainOfCurrentPointer(obj);
            obj.conf.h_pixelinfo.changePatternAndCbk(pixelInfoPattern, cbk);
        end

        function fillSidePanel(obj)
        % GTGRAINSMANAGER/FILLSIDEPANEL Fills the side panel with GUI elements
        % to interact with the data (visualisation, assembling, reloading...)

            % Spacer
            uicontrol('Parent', obj.conf.side_g_boxes, 'Enable', 'off', 'Visible', 'off');

            % Deselected grains button
            obj.conf.h_rec_t_butt = uicontrol('Style', 'pushbutton', ...
                                              'String', 'Reconstruct & Segment', ...
                                              'Parent', obj.conf.side_g_boxes);

            % Spacer
            uicontrol('Parent', obj.conf.side_g_boxes, 'Enable', 'off', 'Visible', 'off');

            % Deselected grains button
            obj.conf.h_desel_butt = uicontrol('Style', 'pushbutton', ...
                                              'String', 'Deselected Grains', ...
                                              'Parent', obj.conf.side_g_boxes);

            % Spacer
            uicontrol('Parent', obj.conf.side_g_boxes, 'Enable', 'off', 'Visible', 'off');

            obj.conf.h_phases_labels = {};
            sample = obj.getSample();
            for phaseID = 1:length(sample.phases)
                obj.conf.h_phases_labels{phaseID} = ...
                            uicontrol('Style', 'text', ...
                                      'String', sprintf('Phase %02d', phaseID), ...
                                      'Parent', obj.conf.side_g_boxes, ...
                                      'HorizontalAlignment', 'left');
            end
            obj.conf.h_volume_label = uicontrol('Style', 'text', ...
                                              'HorizontalAlignment', 'left', ...
                                              'String', 'Complete Volume', ...
                                              'Parent', obj.conf.side_g_boxes);
            obj.conf.h_dil_volume_label = uicontrol('Style', 'text', ...
                                              'HorizontalAlignment', 'left', ...
                                              'String', 'Dilated Volume', ...
                                              'Parent', obj.conf.side_g_boxes);

            % Assemble grains button
            obj.conf.h_assem_butt = uicontrol('Style', 'pushbutton', ...
                                              'String', 'Assemble & D.', ...
                                              'Parent', obj.conf.side_g_boxes);

            % Spacer
            uicontrol('Parent', obj.conf.side_g_boxes, 'Enable', 'off', 'Visible', 'off');

            obj.conf.h_p_vol_butt = uicontrol('Style', 'radiobutton', ...
                                              'String', 'Phase Volume', ...
                                              'Parent', obj.conf.side_g_boxes);

            obj.conf.h_c_vol_butt = uicontrol('Style', 'radiobutton', ...
                                              'String', 'Complete Volume', ...
                                              'Parent', obj.conf.side_g_boxes);

            obj.conf.h_d_vol_butt = uicontrol('Style', 'radiobutton', ...
                                              'String', 'Dilated Volume', ...
                                              'Parent', obj.conf.side_g_boxes);

            obj.conf.h_reload_butt = uicontrol('Style', 'pushbutton', ...
                                              'String', 'Reload Volume', ...
                                              'Parent', obj.conf.side_g_boxes);

            % Spacer
            uicontrol('Parent', obj.conf.side_g_boxes, 'Enable', 'off', 'Visible', 'off');

            obj.conf.h_abs_butt = uicontrol('Style', 'togglebutton', ...
                                              'String', 'Absorption Vol', ...
                                              'Parent', obj.conf.side_g_boxes);

            % Spacer
            uicontrol('Parent', obj.conf.side_g_boxes, 'Enable', 'off', 'Visible', 'off');

            obj.conf.h_query_butt = uicontrol('Style', 'pushbutton', ...
                                              'String', 'Query Metadata', ...
                                              'Parent', obj.conf.side_g_boxes);

            if (isempty(obj.conf.overlay))
                valueAbsVol = get(obj.conf.h_abs_butt, 'Min');
            else
                valueAbsVol = get(obj.conf.h_abs_butt, 'Max');
            end
            set(obj.conf.h_abs_butt, 'Value', valueAbsVol);

            buttonSize = get(obj.conf.side_g_boxes, 'ButtonSize');
            buttonSize(1) = 120;
            set(obj.conf.side_g_boxes, 'ButtonSize', buttonSize);
            set(obj.conf.side_g_boxes, 'VerticalAlignment', 'top');

            obj.checkVolumesUpdate();

            obj.addUICallback(obj.conf.h_rec_t_butt, ...
                              'Callback', ...
                              @(src, evt)launchReconSegGUI(obj), false);

            obj.addUICallback(obj.conf.h_desel_butt, ...
                              'Callback', ...
                              @(src, evt)viewDeselectedGrains(obj), false);

            obj.addUICallback(obj.conf.h_assem_butt, ...
                              'Callback', ...
                              @(src, evt)launchAssembleGUI(obj), false);

            obj.addUICallback(obj.conf.h_p_vol_butt, ...
                              'Callback', ...
                              @(src, evt)setVolumeToLoad(obj, 'phase'), false);

            obj.addUICallback(obj.conf.h_c_vol_butt, ...
                              'Callback', ...
                              @(src, evt)setVolumeToLoad(obj, 'complete'), false);

            obj.addUICallback(obj.conf.h_d_vol_butt, ...
                              'Callback', ...
                              @(src, evt)setVolumeToLoad(obj, 'dilated'), false);

            obj.addUICallback(obj.conf.h_reload_butt, ...
                              'Callback', ...
                              @(src, evt)reloadAssebledVolume(obj), false);

            obj.addUICallback(obj.conf.h_abs_butt, ...
                              'Callback', ...
                              @(src, evt)toggleAbsorptionVolume(obj), false);

            obj.addUICallback(obj.conf.h_query_butt, ...
                              'Callback', ...
                              @(src, evt)showQueryDialog(obj), false);

            obj.addTimer('TimerFcn', @(src, evt)checkVolumesUpdate(obj), ...
                         'Period', 2, 'StartDelay', 1, ...
                         'ExecutionMode', 'fixedSpacing');

            % Bad bodge, but it also creates the internal field 'load_vol'
            obj.setVolumeToLoad('complete');

            drawnow('expose');
        end

        function resetUiComponents(obj)
        % GTGRAINSMANAGER/RESETUICOMPONENTS Inherited method to reset the ui
        % components

            set(obj.conf.h_phase_label, 'String', 'Phase ID (0 = All):');
            set(obj.conf.h_phase_edit, 'String', obj.selectedPhase);
            set(obj.conf.h_grain_label, 'String', 'Grain ID (0 = All):');
            set(obj.conf.h_grain_edit, 'String', obj.selectedGrain);
            if (obj.selectedPhase == 0)
                set(obj.conf.h_grain_edit, 'Enable', 'off');
                set(obj.conf.h_grain_butt, 'Enable', 'off');
            else
                set(obj.conf.h_grain_edit, 'Enable', 'on');
                set(obj.conf.h_grain_butt, 'Enable', 'on');
            end
            obj.updateStatus('Loading..');

            drawnow('expose');

            resetUiComponents@GtVolView(obj);
        end

        function setVisualisedData(obj, image, vol, is_overlay)
        % GTGRAINSMANAGER/SETVISUALISEDDATA Prepares the slices to be visualised
        % in the axes, and then sets them

            slice = obj.loadSlice(vol);

            if (~is_overlay)
                if (~strcmpi(obj.conf.load_vol, 'phase'))
                    % If loading the full volume, let's manage the conflicts and
                    % selections
                    if (obj.selectedPhase > 0)
                        phaseIDs = obj.loadSlice(obj.cache.get('volume', {}, 'phases'));

                        slice(~(phaseIDs == obj.selectedPhase)) = 0;

                        if (obj.selectedGrain > 0)
                            selectedPhaseOffset = ( obj.separateGrainIDs ...
                                                    * (obj.selectedPhase - 1));
                            slice = obj.selectedGrain ...
                                    * (slice == (obj.selectedGrain + selectedPhaseOffset));

                            if (isempty(find(slice ~= 0, 1)))
                                % Empty slice! it means the grain has no volume
                                % or it was not selected! Let's show a contour!
                                slice = obj.getContourOfGrain(obj.selectedPhase, obj.selectedGrain);
                            end
                        elseif (obj.selectedGrain < 0)
                            grainIDs = obj.loadSlice(obj.cache.get('volume', {}, 'grains'));
                            slice(~(grainIDs == -1)) = 0;
                        end
                    elseif (obj.selectedPhase < 0)
                        phaseIDs = obj.loadSlice(obj.cache.get('volume', {}, 'phases'));
                        slice(~(phaseIDs == -1)) = 0;
                    end
                end
            else
                slice = (repmat(slice, [1, 1, 3]) - obj.conf.minOverlayVol) ...
                        / (obj.conf.maxOverlayVol - obj.conf.minOverlayVol);
            end
            set(image, 'cdata', slice);
        end

        function slice = getContourOfGrain(obj, phaseID, grainID)
        % GTGRAINSMANAGER/GETCONTOUROFGRAIN Function that gets border of grains,
        % when grain is not into assembled volume
            sample = obj.getSample();
            vol = obj.cache.get('grain', {phaseID, grainID}, 'seg');
            bbox = sample.phases{phaseID}.getBoundingBox(grainID);

            indx = obj.conf.slicendx - bbox(obj.conf.ortDim);
            if (indx >= 1 && indx <= bbox(obj.conf.ortDim + 3))
                switch (obj.conf.ortDim)
                    case 3 % equivalent to plane 'xy' or 'yx'
                        slice = zeros(obj.conf.dim(1), obj.conf.dim(2));
                        sliceVol = vol(:, :, indx);
                        slice = gtPlaceSubVolume(slice, sliceVol, ([bbox(1) bbox(2) 1]-1));
                    case 1 % equivalent to plane 'yz' or 'zy'
                        slice = zeros(obj.conf.dim(2), obj.conf.dim(3));
                        sliceVol = squeeze(vol(indx, :, :));
                        slice = gtPlaceSubVolume(slice, sliceVol, ([1 bbox(2) bbox(3)]-1));
                    case 2 % equivalent to plane 'zx' or 'xz'
                        slice = zeros(obj.conf.dim(1), obj.conf.dim(3));
                        sliceVol = squeeze(vol(:, indx, :));
                        slice = gtPlaceSubVolume(slice, sliceVol, ([bbox(1) 1 bbox(3)]-1));
                end

                padding1 = zeros(size(slice, 1), 1);
                zer1 = zeros(size(slice, 1)-1, 1);
                D1 = diag(ones(size(slice, 1)-1, 1));
                D1 = [D1, zer1] - [zer1, D1];
                padding2 = zeros(size(slice, 2), 1);
                zer2 = zeros(size(slice, 2)-1, 1);
                D2 = diag(ones(size(slice, 2)-1, 1));
                D2 = [D2, zer2] - [zer2, D2];

                slice = grainID * (([abs(D1 * slice); padding2'] + [abs(D2 * slice')', padding1]) ~= 0);
            else
                switch (obj.conf.ortDim)
                    case 3 % equivalent to plane 'xy' or 'yx'
                        slice = zeros(obj.conf.dim(1), obj.conf.dim(2));
                    case 1 % equivalent to plane 'yz' or 'zy'
                        slice = zeros(obj.conf.dim(2), obj.conf.dim(3));
                    case 2 % equivalent to plane 'zx' or 'xz'
                        slice = zeros(obj.conf.dim(1), obj.conf.dim(3));
                end
            end

            % Cope with matlab showing issue
            if (obj.conf.verDim > obj.conf.horDim)
                slice = permute(slice, [2, 1]);
            end
        end

        function addUICallbacks(obj)
        % GTGRAINSMANAGER/ADDUICALLBACKS Adds the callbacks to the GUI elements

            addUICallbacks@GtVolView(obj)

            obj.addUICallback(obj.conf.h_phase_butt, ...
                              'Callback', ...
                              @(src, evt)updateVisualisedPhase(obj), false);
            obj.addUICallback(obj.conf.h_phase_edit, ...
                              'Callback', ...
                              @(src, evt)updateVisualisedPhase(obj), false);

            obj.addUICallback(obj.conf.h_grain_butt, ...
                              'Callback', ...
                              @(src, evt)updateVisualisedGrain(obj), false);
            obj.addUICallback(obj.conf.h_grain_edit, ...
                              'Callback', ...
                              @(src, evt)updateVisualisedGrain(obj), false);
        end

        function updateStatus(obj, statusString)
        % GTGRAINSMANAGER/UPDATESTATUS Changes the displayed string in the
        % status bar

            set(obj.conf.h_grain_status, 'String', ['Status: ' statusString]);
        end

        function createAxesMenu(obj)
        % GTGRAINSMANAGER/CREATEAXESMENU Creates the context menu in the axes

            createAxesMenu@GtVolView(obj)
            % Built-in Items
            obj.conf.h_menu_items{end+1} = uimenu(obj.conf.h_context_menu, ...
                                           'Label', 'Voxel Details', ...
                                           'Callback', @(src,evt)displayVoxelDetails(obj, get3DPixelInfo(obj)));
            obj.conf.h_menu_items{end+1} = uimenu(obj.conf.h_context_menu, ...
                                           'Label', 'Grain Info', ...
                                           'Callback', @(src,evt)displayGrainInfo(obj, get3DPixelInfo(obj)));
            obj.conf.h_menu_items{end+1} = uimenu(obj.conf.h_context_menu, ...
                                           'Label', 'Select Diffraction Spots', ...
                                           'Callback', @(src,evt)displayDiffspots(obj, get3DPixelInfo(obj)));
            % Extended Items (user plugins)
            for ii = 1:size(obj.plugins, 2)
                disp('adding menuitem')
                obj.conf.h_menu_items{end+1} = uimenu(obj.conf.h_context_menu, ...
                                               'Label', obj.plugins{1,ii}, ...
                                               'Callback', @(src,evt)obj.plugins{2,ii}(obj, get3DPixelInfo(obj)));
            end
        end

        function guiQuit(obj)
        % GTGRAINSMANAGER/GUIQUIT Inherited method to close the manager
            obj.updateStatus('Closing..');

            drawnow('expose');

            sample = obj.getSample();
            if (~isempty(sample))
                try
                    % Redundant but ensures we save it, in case of previous
                    % errors (if we are at least in the correct directory)
                    obj.setSample(sample);
                catch mexc
                    gtPrintException(mexc)
                    d = warndlg(mexc.message);
                    uiwait(d, 10);
                end
            end

            guiQuit@GtVolView(obj);
        end

        function setSample(obj, sample)
            fprintf('Saving to file..')
            obj.cache.set('sample', {}, sample, 'sample');
            obj.cache.flushObjectField('sample', {}, 'sample');
            fprintf(' Done.\n')
        end

        function sample = getSample(obj)
            sample = obj.cache.get('sample', {}, 'sample');
        end

        function phasePanel = makePhasePanel(obj, parent, phaseID, grainIDs, varargin)
        % GTGRAINSMANAGER/MAKEPHASEPANEL Creates the phase panel, which contains
        % all the grain panels of the selected grain for the calling action.

            phasePanel = [];
            phasePanel.panel = uipanel('Parent', parent, varargin{:});
            phasePanel.scroll_boxes = uiextras.HBox('Parent', phasePanel.panel);

            phasePanel.phase_boxes = uiextras.VBox('Parent', phasePanel.scroll_boxes);
            phasePanel.scroll_bar = uicontrol('Parent', phasePanel.scroll_boxes, ...
                                              'Style', 'slider');

            set(phasePanel.scroll_boxes, 'Sizes', [-1 20]);

            numGrainsFound = length(grainIDs);
            phaseString = sprintf('Phase %02d: %04d grains', phaseID, numGrainsFound);
            phasePanel.phase_label = uicontrol('Parent', phasePanel.phase_boxes, ...
                                              'Style', 'text', ...
                                              'String', phaseString);
            fprintf('%s\n', phaseString)
            disp(grainIDs)

            phasePanel.grainPanels = {};
            for ii = 1:numGrainsFound
                grainID = grainIDs(ii);
                phasePanel.grainPanels{end+1} = obj.makeGrainPanel(...
                                    phasePanel.phase_boxes, phaseID, grainID);
            end

            sizesVec = ones(1, numGrainsFound+1) * (-1);
            sizesVec(1) = 20;
            set(phasePanel.phase_boxes, 'Sizes', sizesVec);

            % Record the size of a grain panel, for scrolling
            pixels = GtBaseGuiElem.getPixels(phasePanel.grainPanels{1}.descr, 'Extent');
            phasePanel.panelsSize = 20 + pixels(4);

            drawnow('expose');

            % Add resize callback
            set(phasePanel.panel, 'ResizeFcn', @(src, evt)resizePhasePanel(obj, phasePanel));
            set(phasePanel.scroll_bar, 'Callback', @(src, evt)updatePhasePanelSlider(obj, phasePanel));

            obj.resizePhasePanel(phasePanel);
        end

        function resizePhasePanel(~, phasePanel)
        % GTGRAINSMANAGER/RESIZEPHASEPANEL Manages the visibility of the grain
        % panels into a phase panel, in order to make them appear completely
        % even if the phase panel is not big enough to containe them all

            pixels = GtBaseGuiElem.getPixels(phasePanel.panel, 'Position');
            numOfPanels = length(phasePanel.grainPanels);
            if (pixels(4) < (phasePanel.panelsSize * numOfPanels + 20))
                set(phasePanel.scroll_bar, 'Enable', 'on');
                set(phasePanel.scroll_bar, 'Visible', 'on');

                numVisible = max(int32(floor((pixels(4) - 20)/phasePanel.panelsSize)), 1);

                sizesVec = zeros(1, numOfPanels+1);
                sizesVec(1) = 20;
                sizesVec(2:numVisible+1) = -1;
                set(phasePanel.phase_boxes, 'Sizes', sizesVec);

                set(phasePanel.scroll_bar, 'Min', 1);
                set(phasePanel.scroll_bar, 'Max', numOfPanels);
                set(phasePanel.scroll_bar, 'Value', numOfPanels);
                sliderStep = 1/double(max(numOfPanels-1, 1));
                set(phasePanel.scroll_bar, 'SliderStep', [sliderStep sliderStep]);
            else
                set(phasePanel.scroll_bar, 'Enable', 'off');
                set(phasePanel.scroll_bar, 'Visible', 'off');

                sizesVec = ones(1, numOfPanels+1) * (-1);
                sizesVec(1) = 20;
                set(phasePanel.phase_boxes, 'Sizes', sizesVec);
            end

            drawnow('expose');
        end

        function updatePhasePanelSlider(~, phasePanel)
        % GTGRAINSMANAGER/UPDATEPHASEPANELSLIDER If triggered updates the
        % visibility of the grain panels, accordin to the position of the slider

            pixels = GtBaseGuiElem.getPixels(phasePanel.panel, 'Position');
            numOfPanels = length(phasePanel.grainPanels);
            numVisible = max(int32(floor((pixels(4) - 20)/phasePanel.panelsSize)), 1);
            base = int32(numOfPanels - get(phasePanel.scroll_bar, 'Value')+1);
            last = min(numVisible+base, numOfPanels+1);

            sizesVec = zeros(1, numOfPanels+1);
            sizesVec(1) = 20;
            sizesVec(1+base:last) = -1;
            set(phasePanel.phase_boxes, 'Sizes', sizesVec);
        end

        function grainPanel = makeGrainPanel(obj, parent, phaseID, grainID, varargin)
        % GTGRAINSMANAGER/MAKEGRAINPANEL Generates a panel which will fit into a
        % GUI and contain the information about a grain, and all the buttons to
        % interact with it.

            grainPanel = [];
            grainPanel.panel = uipanel('Parent', parent, varargin{:});
            grainPanel.binfo = uiextras.VBox('Parent', grainPanel.panel);
            grainPanel.boxes = uiextras.HBox('Parent', grainPanel.binfo);
            grainPanel.label = uicontrol('Parent', grainPanel.boxes, ...
                                         'Style', 'text', ...
                                         'HorizontalAlignment', 'left', ...
                                         'String', sprintf('Grain: %04d', grainID));
            grainPanel.select = uicontrol('Parent', grainPanel.boxes, ...
                                         'Style', 'togglebutton');
            grainPanel.difspt = uicontrol('Parent', grainPanel.boxes, ...
                                         'Style', 'pushbutton', ...
                                         'String', 'Diffspots');
            grainPanel.reconstr = uicontrol('Parent', grainPanel.boxes, ...
                                         'Style', 'pushbutton', ...
                                         'String', 'Reconstruct');
            grainPanel.thresh = uicontrol('Parent', grainPanel.boxes, ...
                                         'Style', 'pushbutton', ...
                                         'String', 'Threshold');
            grainPanel.see = uicontrol('Parent', grainPanel.boxes, ...
                                         'Style', 'pushbutton', ...
                                         'String', 'Visualize Seg');
            grainPanel.fsim = uicontrol('Parent', grainPanel.boxes, ...
                                         'Style', 'pushbutton', ...
                                         'String', 'ForwardSim');
            grainPanel.pdesc = uipanel('Parent', grainPanel.binfo);
            grainPanel.descr = uicontrol('Parent', grainPanel.pdesc, ...
                                         'Style', 'text', ...
                                         'Units', 'normalized', ...
                                         'Position', [0 0 1 1], ...
                                         'HorizontalAlignment', 'left', ...
                                         'String', obj.produceGrainInfo(phaseID, grainID));
            set(grainPanel.binfo, 'Sizes', [20 -1]);
            set(grainPanel.boxes, 'Sizes', [-1 80 70 85 75 95 80]);

            sample = obj.getSample();
            % Activeness
            isSelected = sample.phases{phaseID}.getSelected(grainID);
            if (isSelected)
                set(grainPanel.select, 'String', 'Active');
                set(grainPanel.select, 'Value', get(grainPanel.select, 'Max'))
            else
                set(grainPanel.select, 'String', 'Not Active');
                set(grainPanel.select, 'Value', get(grainPanel.select, 'Min'))
            end

            drawnow('expose');

            % Adding callbacks to buttons
            set(grainPanel.select, 'Callback', @(src,evt)toggleSelected(obj, src, phaseID, grainID));
            set(grainPanel.difspt, 'Callback', @(src,evt)selectDiffSpots(obj, phaseID, grainID));
            set(grainPanel.reconstr, 'Callback', @(src,evt)reconstructGrain(obj, phaseID, grainID));
            set(grainPanel.thresh, 'Callback', @(src,evt)changeGrainThreshold(obj, phaseID, grainID));
            set(grainPanel.see, 'Callback', @(src,evt)viewSegmentedGrain(obj, phaseID, grainID));
            set(grainPanel.fsim, 'Callback', @(src,evt)displayForwardSim(obj, phaseID, grainID));
        end

        function infoStr = produceGrainInfo(obj, phaseNum, grainNum)
        % GTGRAINSMANAGER/PRODUCEGRAININFO Produces a string which contains the
        % info about a specific grain in a specific phase.

            sample = obj.getSample();
            phase = sample.phases{phaseNum};
            infoStr = [];
            infoStr = [infoStr sprintf('Phase ID:     %d\n', phaseNum)];
            infoStr = [infoStr sprintf('Grain ID:     %d\n', grainNum)];
            infoStr = [infoStr sprintf('Is selected:  %d\n', phase.getSelected(grainNum))];
            infoStr = [infoStr sprintf('Center:       (%f, %f, %f)\n', phase.getCenter(grainNum))];
            infoStr = [infoStr sprintf('Bounding box: pos (%d, %d, %d), dims (%d, %d, %d)\n', phase.getBoundingBox(grainNum))];
            infoStr = [infoStr sprintf('Threshold:    %d\n', phase.getThreshold(grainNum))];
            infoStr = [infoStr sprintf('Completeness: %d\n', phase.getCompleteness(grainNum))];
            infoStr = [infoStr sprintf('R vector:     (%f, %f, %f)\n', phase.getR_vector(grainNum))];
%             if (phase.isConflictInformationAvailable())
%                 if ( phase.getHasConflicts(grainNum) )
%                     infoStr = [infoStr sprintf('Has conflicts?                       YES\n')];
%                     numTotDiffspots = length(unique(phase.getDifspotID(grainNum)));
%                     numConflictDiffspots = length(unique(phase.getIdSpotIds(grainNum)));
%                     infoStr = [infoStr sprintf('Difspots from Forward Simulation:    %d\n', numTotDiffspots)];
%                     infoStr = [infoStr sprintf('...in common with other grains:      %d\n', numConflictDiffspots)];
%                     for jj = 1 : phase.getNumGrain(grainNum)
%                         tmp = phase.getConflicts(grainNum, jj);
%                         infoStr = [infoStr sprintf('Conflict ID: %d      Difspots in common with grainID %d: %d\n', tmp.difspots{1}, grainNum, length(tmp.difspots{2}))];
%                     end
%                 else
%                     infoStr = [infoStr sprintf('Has conflicts?    NO\n')];
%                 end
%             else
%                 infoStr = [infoStr sprintf('No "conflict information" available\n')];
%             end

            drawnow('expose');
        end

        function [upToDate, phaseDateNum] = isPhaseUptodate(obj, phaseID)
        % GTGRAINSMANAGER/ISPHASEUPTODATE Checks if the phase volume denoted by
        % phaseID is up to date in respect to the grains that compose it.

            sample = obj.getSample();
            phase = sample.phases{phaseID};
            % Phase datetime
            volumeFile = regexp(phase.volumeFile, ':', 'split');
            volumeFile = fullfile(obj.dataDir, volumeFile{1});
            if (isempty(volumeFile) || ~exist(volumeFile, 'file'))
                upToDate = false;
                phaseDateNum = 0;
                return
            else
                upToDate = true;
                fileInfo = dir(volumeFile);
                phaseDateNum = fileInfo(1).datenum;
            end

            filePath = fullfile(obj.dataDir, '4_grains', ...
                                sprintf('phase_%02d', phaseID), 'grain_*.mat');
            fileInfo = dir(filePath);
            numGrains = length(phase.grains);
            if (length(fileInfo) < numGrains)
                upToDate = false;
                return
            else
                grainDateNums = [fileInfo.datenum];
                if ( any(grainDateNums > phaseDateNum) )
                    upToDate = false;
                    return
                end
            end
        end

        function panelBlk = produceQueryPanel(obj, parent)
            panelBlk = [];
            panelBlk.d = uiextras.VBox('Parent', parent);
            panelBlk.query_boxes = uiextras.VBox('Parent', panelBlk.d);
            panelBlk.result_panel = uipanel('Parent', panelBlk.d);

            set(panelBlk.d, 'Sizes', [50, -1]);

            panelBlk.h_query = [];
            panelBlk.h_query.completeness = [];
            panelBlk.h_query.completeness.b = uiextras.HBox('Parent', panelBlk.query_boxes);
            panelBlk.h_query.difspots = [];
            panelBlk.h_query.difspots.b = uiextras.HBox('Parent', panelBlk.query_boxes);

            panelBlk.h_query.completeness.label = uicontrol('Parent', panelBlk.h_query.completeness.b, ...
                                            'Style', 'text', ...
                                            'HorizontalAlignment', 'left', ...
                                            'String', 'List the grains that have completeness lower than:');
            panelBlk.h_query.completeness.edit = uicontrol('Parent', panelBlk.h_query.completeness.b, ...
                                            'Style', 'edit');
            panelBlk.h_query.completeness.butt = uicontrol('Parent', panelBlk.h_query.completeness.b, ...
                                            'Style', 'pushbutton', ...
                                            'String', 'Query');
            set(panelBlk.h_query.completeness.b, 'Sizes', [-1 50 50]);

            panelBlk.h_query.difspots.label = uicontrol('Parent', panelBlk.h_query.difspots.b, ...
                                            'Style', 'text', ...
                                            'HorizontalAlignment', 'left', ...
                                            'String', 'List the grains that have fewer difspots than:');
            panelBlk.h_query.difspots.edit = uicontrol('Parent', panelBlk.h_query.difspots.b, ...
                                            'Style', 'edit');
            panelBlk.h_query.difspots.butt = uicontrol('Parent', panelBlk.h_query.difspots.b, ...
                                            'Style', 'pushbutton', ...
                                            'String', 'Query');
            set(panelBlk.h_query.difspots.b, 'Sizes', [-1 50 50]);

            set(panelBlk.h_query.completeness.butt, 'Callback', ...
                @(src, evt)launchQuery(obj, panelBlk, 'completeness'))
            set(panelBlk.h_query.completeness.edit, 'Callback', ...
                @(src, evt)launchQuery(obj, panelBlk, 'completeness'))
            set(panelBlk.h_query.difspots.butt, 'Callback', ...
                @(src, evt)launchQuery(obj, panelBlk, 'difspots'))
            set(panelBlk.h_query.difspots.edit, 'Callback', ...
                @(src, evt)launchQuery(obj, panelBlk, 'difspots'))

            drawnow('expose');
        end
    end

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Utility Methods independent of the visualisation object
    methods (Static, Access = public)
        function [paramsOut, itemsNum] = checkMenuParams(params)
        % GTGRAINSMANAGER/CHECKMENUPARAMS Checks wether the menu plugins are
        % correctly passed or not

            if (~iscell(params))
                error('PARAMS:no_cell_array', ...
                      [ 'The context menu additional parametes are supposed' ...
                        ' to be in a cell array']);
            end
            if (mod(length(params), 2) == 1)
                error('PARAMS:wrong_parameters_number', ...
                      [ 'The context menu additional parametes are supposed' ...
                        ' to be in pairs: "menu string", @function_handle']);
            end

            itemsNum = uint64(length(params)/2);
            paramsOut = cell(2, itemsNum);
            paramsOut(1,1:end) = params(1:2:end);
            paramsOut(2,1:end) = params(2:2:end);

            for ii = 1:length(paramsOut(1,1:end))
                if (~ischar(paramsOut{1,ii}))
                    error('PARAMS:wrong_parameters_type', ...
                          [ 'The context menu additional parametes are ' ...
                            'supposed to have strings as menu item labes']);
                end
            end
            for ii = 1:length(paramsOut(2,1:end))
                if (~isa(paramsOut{2,ii}, 'function_handle'))
                    error('PARAMS:wrong_parameters_type', ...
                          [ 'The context menu additional parametes are ' ...
                            'supposed to function handles as callbacks']);
                end
            end
        end
    end
end
